/*
 *  Copyright (C) 2004 Thecus Technology Corp. 
 *
 *      Written by Y.T. Lee (yt_lee@thecus.com)
 *
 *      based on linux/drivers/i2c/chips/rtc8564.c
 *
 *  Copyright (C) 2002-2004 Stefan Eletzhofer
 *
 *	based on linux/drivers/acron/char/pcf8583.c
 *  Copyright (C) 2000 Russell King
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Driver for Ricoh RTC RS372 chip on Thecus N4100
 */
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/i2c.h>
#include <linux/slab.h>
#include <linux/string.h>
#include <linux/rtc.h>		/* get the user-level API */
#include <linux/init.h>
#include <linux/bcd.h>
#include <linux/types.h>
#include <linux/miscdevice.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/rtc.h>
#include <linux/poll.h>
#include <linux/proc_fs.h>
#include <linux/smp_lock.h>
#include <linux/delay.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/system.h>
#include <asm/arch/iop321.h>
#include <asm/hardware.h>
//#include <asm/mach-types.h>
#include <linux/interrupt.h>
#include <asm/irq.h>
#include <asm/mach/irq.h>
#include <linux/seq_file.h>
#include "thecus_rtc.h"
#include <linux/notifier.h>

// #define DEBUG 1
#define SET_FAN2 1
#define I2C_RETRY 5
#define ACCESS_RTC 1

#ifdef DEBUG
# define _DBG(x, fmt, args...) do{ if (debug>=x) printk(KERN_DEBUG"%s: " fmt "\n", __FUNCTION__, ##args); } while(0);
#else
# define _DBG(x, fmt, args...) do { } while(0);
#endif

#define _DBGRTCTM(x, rtctm) if (debug>=x) printk("%s: secs=%d, mins=%d, hours=%d, mday=%d, " \
			"mon=%d, year=%d, wday=%d\n", __FUNCTION__, \
			(rtctm).secs, (rtctm).mins, (rtctm).hours, (rtctm).mday, \
			(rtctm).mon, (rtctm).year, (rtctm).wday);

MODULE_AUTHOR("Stefan Eletzhofer <Stefan.Eletzhofer@eletztrick.de>");
MODULE_DESCRIPTION("Ricoh(Thecus) RTC RS372 Driver");
MODULE_LICENSE("GPL");
static int HPI = 0;
static int debug;
static int TRIM=0x1;
module_param(TRIM, int, S_IRUGO | S_IWUSR);
module_param(debug, int, S_IRUGO | S_IWUSR);
module_param(HPI, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(HPI, "HPI enable");

struct rtc_rs372_data {
	struct i2c_client client;
	u16 ctrl;
};

struct pca_9532_data {
	struct i2c_client client;
	u8 ctrl;
};

struct f75373_data {
	struct i2c_client client;
	u8 ctrl;
};

static int XSTP=0;

static struct i2c_client *rs372;
static struct i2c_client *pca9532;
static struct i2c_client *f75373;

static char kmsgs[80];

static inline u8 _rtc_rs372_ctrl1(struct i2c_client *client)
{
	struct rtc_rs372_data *data = i2c_get_clientdata(client);
	return data->ctrl & 0xff;
}
static inline u8 _rtc_rs372_ctrl2(struct i2c_client *client)
{
	struct rtc_rs372_data *data = i2c_get_clientdata(client);
	return (data->ctrl & 0xff00) >> 8;
}

extern spinlock_t rtc_lock;
static int rtc_ioctl(struct inode *inode, struct file *file,
		     unsigned int cmd, unsigned long arg);

static int rtc_read_proc(char *page, char **start, off_t off,
                         int count, int *eof, void *data);

//static void get_rtc_time(struct rtc_time *rtc_tm);
/*
 *	Bits in rtc_status. (6 bits of room for future expansion)
 */

#define RTC_IS_OPEN		0x01	/* means /dev/rtc is in use	*/
#define RTC_TIMER_ON		0x02	/* missed irq timer active	*/

#define RS372_I2C_ID  0x32
#define PCA9532_I2C_ID 0x60
#define F75373_I2C_ID 0x2e

u8 f75375_slave_addr=F75373_I2C_ID;


static unsigned char rtc_status;	/* bitmapped status byte.	*/

/*
 *	If this driver ever becomes modularised, it will be really nice
 *	to make the epoch retain its value across module reload...
 */

static unsigned long epoch = 1970;	/* year corresponding to 0x00	*/

static const unsigned char days_in_mo[] =
{0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};


#define CTRL1(c) _rtc_rs372_ctrl1(c)
#define CTRL2(c) _rtc_rs372_ctrl2(c)

static struct i2c_driver rtc_rs372_driver;
static struct i2c_driver PCA_9532_driver;
static struct i2c_adapter *pca9532_adaptor;
static struct i2c_driver f75373_driver;
static struct i2c_adapter *f75373_adaptor;

static unsigned short ignore[] = { I2C_CLIENT_END };
static unsigned short normal_addr[] = { RS372_I2C_ID, I2C_CLIENT_END };
static unsigned short pca_9532_addr[] = { PCA9532_I2C_ID, I2C_CLIENT_END };
static unsigned short f75373_addr[] = { F75373_I2C_ID, I2C_CLIENT_END };

static struct i2c_client_address_data addr_data = {
	.normal_i2c		= normal_addr,
	.probe			= ignore,
	.ignore			= ignore,
};

static struct i2c_client_address_data pca_9532_addr_data = {
	.normal_i2c		= pca_9532_addr,
	.probe			= ignore,
	.ignore			= ignore,
};

static struct i2c_client_address_data f75373_addr_data = {
	.normal_i2c		= f75373_addr,
	.probe			= ignore,
	.ignore			= ignore,
};

static int rtc_rs372_read_mem(struct i2c_client *client, struct mem *mem);
static int rtc_rs372_write_mem(struct i2c_client *client, struct mem *mem);
static int pulse=1;

/* Input: BIT#, action=0/1 */
static void gpio_write(int gpio_bit,int action)
{
  unsigned char gpio_byte,out_byte;
  out_byte = *IOP321_GPID;
  gpio_byte = (1 << gpio_bit);
  if(action==0){ //input 0: want to turn off
    out_byte |= gpio_byte;
  }
  else{
    gpio_byte ^= 0xFF;
    out_byte &= gpio_byte;
  }
  *IOP321_GPOD = out_byte;

}
/* return 0 for no error */
static int pca_9532_rw(struct i2c_adapter *adap, u8 reg_num, u8 *val, int wr)
{
	int ret;
	unsigned char ad[1];
	unsigned char data[2];
	struct i2c_msg ctrl_wr[1] = {
	  {PCA9532_I2C_ID, 0, 2, data}
	};
	struct i2c_msg ctrl_rd[2] = {
		{PCA9532_I2C_ID, 0, 1, ad},
		{PCA9532_I2C_ID, I2C_M_RD, 1, data}
	};

        if(NULL == adap) {
            printk(KERN_INFO "thecus_rtc: i2c_adapter is NULL\n");
            return 1;
        }

	if(wr){ //Write
	  data[0]=reg_num;
	  data[1]=*val;

	  ret = i2c_transfer(adap, ctrl_wr, 1);
	  if (ret != 1) {
	    printk(KERN_INFO "thecus_rtc: cant write PCA9532 Reg#%02X\n",reg_num);
	    return ret;
	  }
	}
	else{ //Read

	  data[0]=0;
	  data[1]=0;
	  ad[0]=reg_num;
	  ret = i2c_transfer(adap, ctrl_rd, 2);
	  if (ret != 2) {
	    printk(KERN_INFO "thecus_rtc: cant read PCA9532 Reg#%02X\n",reg_num);
	    return ret;
	  }
	  *val=data[0];
	}
	return 0;
}

static int pca_9532_attach(struct i2c_adapter *adap, int addr, int kind)
{
	int ret;
	struct i2c_client *new_client;
	struct pca_9532_data *d;
	unsigned char data[16];
	unsigned char ad[1] = { 0x03 };
	struct i2c_msg ctrl_rd[2] = {
		{addr, 0, 1, ad},
		{addr, I2C_M_RD, 1, data}
	};

	d = kzalloc(sizeof(struct pca_9532_data), GFP_KERNEL);
	if (!d) {
		ret = -ENOMEM;
		goto done;
	}
	new_client = &d->client;

	strlcpy(new_client->name, "PCA_9532", I2C_NAME_SIZE);
	i2c_set_clientdata(new_client, d);
	new_client->addr = addr;
	new_client->adapter = adap;
	new_client->driver = &PCA_9532_driver;

	_DBG(1, "client=%p", new_client);

	/* read back ctrl1 and ctrl2 */
	gpio_write( GPIO_LED_RESET, 0);
	mdelay(50);
	gpio_write( GPIO_LED_RESET, 1);

	data[0]=0;
	data[1]=0;
	ad[0]=0x03;
	ret = i2c_transfer(new_client->adapter, ctrl_rd, 2);

	if (ret != 2) {
	  printk(KERN_INFO "pca_9532_attach: cant read ctrl\n");
	  ret = -ENODEV;
	  goto done;
	}

	if(data[0]!=0x80){ //PCA9532 not found
	  pca9532_adaptor=0;
	  ret = -ENODEV;
	  goto done;
	}
	pca9532_adaptor=adap;
	d->ctrl = data[0];

	_DBG(1, "PCA9532_REG_PWM1=%02x",data[0]);

	/*
	data[0]=0x02;
	data[1]=128;

	ret = i2c_transfer(new_client->adapter, ctrl_wr, 1);
	if (ret != 1) {
		printk(KERN_INFO "thecus_rtc: cant write PSC0\n");
		ret = -ENODEV;
		goto done;
	}

	data[0]=0x03;
	data[1]=30;

	ret = i2c_transfer(new_client->adapter, ctrl_wr, 1);
	if (ret != 1) {
		printk(KERN_INFO "thecus_rtc: cant write PWM0\n");
		ret = -ENODEV;
		goto done;
	}


	data[0]=0x06;
      	data[1]=0x55;
	//	data[1]=0x28;

	ret = i2c_transfer(new_client->adapter, ctrl_wr, 1);
	if (ret != 1) {
		printk(KERN_INFO "thecus_rtc: cant write LS0\n");
		ret = -ENODEV;
		goto done;
	}
	data[0]=0x09;
	//	data[1]=0x14;
	data[1]=0x28;

	ret = i2c_transfer(new_client->adapter, ctrl_wr, 1);
	if (ret != 1) {
		printk(KERN_INFO "thecus_rtc: cant write LS3\n");
		ret = -ENODEV;
		goto done;
	}
	data[0]=0;
	data[1]=0;
	ad[0]=0x09;
	ret = i2c_transfer(new_client->adapter, ctrl_rd, 2);
	if (ret != 2) {
		printk(KERN_INFO "thecus_rtc: cant read LS0\n");
		ret = -ENODEV;
		goto done;
	}


	_DBG(1, "PCA9532_REG_LS3=%02x",data[0]);
	*/
	
	pca9532=new_client;
	pca9532_adaptor=adap;
	ret = i2c_attach_client(new_client);
done:
	if (ret) {
		kfree(d);
	}
	return ret;
}

static int pca_9532_probe(struct i2c_adapter *adap)
{
  int ret;
  printk("Probing pca9532\n");
  ret=i2c_probe(adap, &pca_9532_addr_data, pca_9532_attach);
  printk("Probe result=%X\n",ret);
  if(ret != 0) {
    ret=i2c_probe(adap, &pca_9532_addr_data, pca_9532_attach);
    printk("Probe result=%X\n",ret);
  }
  return ret;
}

static int pca_9532_detach(struct i2c_client *client)
{
	i2c_detach_client(client);
	kfree(i2c_get_clientdata(client));
	return 0;
}

static int
pca_9532_command(struct i2c_client *client, unsigned int cmd, void *arg)
{

	_DBG(1, "cmd=%d", cmd);

	switch (cmd) {
	default:
		return -EINVAL;
	}
}

static int
rtc_notify_reboot(struct notifier_block *nb, unsigned long event, void *p)  //added by jones at 2005/07/26
{
        u8 val1=LED_ON, val2;

        switch(event) {
        case SYS_HALT:
        case SYS_POWER_OFF:
		if( pca9532_adaptor != NULL) {
		printk("Shutdown by pca9532\n");
                pca_9532_rw( pca9532_adaptor, LED_LS2, &val2, 0);
                val2&=~(0x3);
                val1|=val2;
                pca_9532_rw( pca9532_adaptor, LED_LS2, &val1, 1);
		}
        }
        return NOTIFY_DONE;
}

struct notifier_block rtc_notifier_reboot = {
        .notifier_call  = rtc_notify_reboot,
        .next           = NULL,
        .priority       = 0
};

/* return 0 for no error */
static int f75373_rw(struct i2c_adapter *adap, u8 reg_num, u8 *val, int wr)
{
	int ret=0;
	unsigned char ad[1];
	unsigned char data[2];
	int i,try_time=I2C_RETRY;
	
	struct i2c_msg ctrl_wr[1] = {
	  {f75375_slave_addr, 0, 2, data}
	};
	struct i2c_msg ctrl_rd[2] = {
		{f75375_slave_addr, 0, 1, ad},
		{f75375_slave_addr, I2C_M_RD, 1, data}
	};

	//printk("f75375_slave_addr=%02X \n",f75375_slave_addr);
	if(NULL == adap) {
	    printk(KERN_INFO "thecus_rtc: i2c_adapter is NULL\n");
	    return 1;
	}

  for (i=0;i<try_time;i++) {
	  if(wr){ //Write
	    data[0]=reg_num;
	    data[1]=*val;
    
	    ret = i2c_transfer(adap, ctrl_wr, 1);
	    if (ret == 1) {
	    	if (i>0) printk("f75373_rw Write try %d time \n",i+1);
	  		break;
	  	} else {
#if 1
	  		if (f75375_slave_addr==0x2d) f75375_slave_addr=0x2e;
	  		else  f75375_slave_addr=0x2d;
	  		ctrl_wr[0].addr=f75375_slave_addr;
	  		f75373_addr[0]=f75375_slave_addr;
	  		printk("In Write Mode Addr Change to 0x%02X  \n",f75375_slave_addr);
#endif	  		
	  	}
	  }
	  else{ //Read
	    data[0]=0;
	    data[1]=0;
	    ad[0]=reg_num;
	    ret = i2c_transfer(adap, ctrl_rd, 2);
	    if (ret == 2) {
	    	if (i>1) printk("f75373_rw Read try %d time \n",i+1);
	    	*val=data[0];
	    	break;
	  	} else {
#if 1
	  		if (f75375_slave_addr==0x2d) f75375_slave_addr=0x2e;
	  		else  f75375_slave_addr=0x2d;
	  		ctrl_rd[0].addr=f75375_slave_addr;
	  		ctrl_rd[1].addr=f75375_slave_addr;
	  		f75373_addr[0]=f75375_slave_addr;
	  		printk("In Read Mode Addr Change to 0x%02X  \n",f75375_slave_addr);
#endif	  		
	  	}
	  }
	}
  if(wr){ //Write
    if (ret != 1) {
      printk(KERN_INFO "thecus_rtc: cant write F75373 Reg#0x%02X error (%d)\n",reg_num,ret);
      return ret;
    }
  }
  else{ //Read
    if (ret != 2) {
      printk(KERN_INFO "thecus_rtc: cant read F75373 Reg#0x%02X error (%d)\n",reg_num,ret);
      return ret;
    }
  }
	//msleep(2);
	return 0;
}

static int f75373_attach(struct i2c_adapter *adap, int addr, int kind)
{
	int ret;
	int i,try_time=I2C_RETRY;
	struct i2c_client *new_client;
	struct f75373_data *d;
	unsigned char data[16];
	unsigned char ad[1] = { 0x03 };
	struct i2c_msg ctrl_rd[2] = {
		{addr, 0, 1, ad},
		{addr, I2C_M_RD, 1, data}
	};

	printk("Allocating memory for F75373\n");
	d = kzalloc(sizeof(struct f75373_data), GFP_KERNEL);
	if (!d) {
		ret = -ENOMEM;
		goto done;
	}
	new_client = &d->client;

	strlcpy(new_client->name, "F75373", I2C_NAME_SIZE);
	i2c_set_clientdata(new_client, d);
	new_client->addr = addr;
	new_client->adapter = adap;
	new_client->driver = &f75373_driver;

	printk("client=%p ", new_client);

	printk("Attaching F75373\n");

	data[0]=0;
	data[1]=0;
	ad[0]=0x5D;
	
	f75373_adaptor=adap;
	for (i=0;i<try_time;i++) {
		ret = i2c_transfer(new_client->adapter, ctrl_rd, 2);
		if (ret == 2) {
			if (i>0) printk("f75373_attach try %d time \n",i+1);
			break;
		} else {
			printk("f75373_attach new_client->adapter=%lX try %d time ret=%02X ,fail ..\n",new_client->adapter,i+1,ret);
		}
	}
	if (ret != 2) {
	  printk(KERN_INFO "f75373_attach: cant read ctrl\n");
	 	sprintf(kmsgs,"Cannot make i2c_transfer\n");
	 	ret = -ENODEV;
	 	goto done;
	}
	printk("Data[0]=0x%02X\n",data[0]);
	/*
	if(data[0]!=0x19){ //Vendor ID not found
	  f75373_adaptor=0;
	  ret = -ENODEV;
	  goto done;
	}
	*/
	d->ctrl = data[0];

	printk("F75373 Vendor ID=%02x",data[0]);


	f75373=new_client;
	f75373_adaptor=adap;
	ret = i2c_attach_client(new_client);
done:
	if (ret) {
		kfree(d);
	}
	return ret;
}

static int f75373_probe(struct i2c_adapter *adap)
{
  int ret=1;
  u8 val1=0,val2=0,val3=0;
  int retry_count;
  printk("Probing F75373\n");

  f75373_adaptor=0;
  retry_count=0;
  while ((ret != 0) && (retry_count < I2C_RETRY)) {
    ret=i2c_probe(adap, &f75373_addr_data, f75373_attach);
    retry_count++;
    printk("F75373 Try count %d ,Probe result=%X\n",retry_count,ret);
    if (ret==0) {
    	//f75375_slave_addr=F75373_I2C_ID;
    	printk("F75373 Time %d rtc_rs372_probe try success !! \n",retry_count);
#if 1   
    } else {
    	if (retry_count%2==1) f75375_slave_addr=0x2D;
			else f75375_slave_addr=0x2E;
    	printk("F75373 origan addr is %02X  ,now use 0x%02X \n",f75373_addr[0],f75375_slave_addr);
    	f75373_addr[0]=f75375_slave_addr;
#endif    	
  	}
  }
  //printk("f75373_adaptor=%lx \n",f75373_adaptor); 
  //ret=f75373_attach(adap,F75373_I2C_ID,0);
  //f75373_adaptor=adap;

  val1=0xF0;
  f75373_rw( adap,val1 , &val2, 0);
  printk("Read F75375 register 0x%02X value = 0x%02X \n",val1,val2);
#if 0
  val2=val2 | 0x2;
#else
  val2=0x02;
#endif  
  printk("Set F75375 register 0x%02X value to 0x%02X  \n",val1,val2);
  f75373_rw( adap,val1 , &val2, 1);
  //read again ...
  f75373_rw( adap,val1 , &val3, 0);
  printk("Read F75375 register 0x%02X value = 0x%02X \n",val1,val3);

  retry_count=1;
  while ((!(val3|0x02))&&(retry_count < I2C_RETRY)) {
  	//Try again
	  printk("Set F75375 register 0x%02X value to 0x%02X  again! \n",val1,val2);
  	f75373_rw( adap,val1 , &val2, 1);
  	retry_count++;
  }
  
#if 0  
  
  val1=0x4;
  val2=0xA9;
  f75373_rw( adap,val1 , &val2, 1);
  val2=0xC3;
  f75373_rw( adap,val1 , &val2, 1);
  val2=f75373_addr[0]<<1;
  printk("Set F75373 Control Address is 0x%02X  \n",val2);
  f75373_rw( adap,val1 , &val2, 1);
#endif
  return ret;
}

static int f75373_detach(struct i2c_client *client)
{
	i2c_detach_client(client);
	kfree(i2c_get_clientdata(client));
	return 0;
}

static int
f75373_command(struct i2c_client *client, unsigned int cmd, void *arg)
{

	_DBG(1, "cmd=%d", cmd);

	switch (cmd) {
	default:
		return -EINVAL;
	}
}

static int rtc_rs372_read(struct i2c_client *client, unsigned char adr,
			unsigned char *buf, unsigned char len)
{
	int ret = -EIO;
	unsigned char addr[1] = { adr };
	int i;
	int try_time=I2C_RETRY;
	struct i2c_msg msgs[2] = {
		{client->addr, 0, 1, addr},
		{client->addr, I2C_M_RD, len, buf}
	};
	addr[0] = adr << 4;

	_DBG(1, "client=%p, adr=%x, buf=%p, len=%d", client, addr[0], buf, len);
	//printk("rtc_rs372_read client=%p, adr=%x, buf=%p, len=%d", client, addr[0], buf, len);

	if (!buf) {
		ret = -EINVAL;
		goto done;
	}

        if(NULL == client->adapter) {
            	printk(KERN_INFO "thecus_rtc: i2c_adapter is NULL\n");
		ret = -EINVAL;
		goto done;
        }


  for (i=0;i<try_time;i++) {
		ret = i2c_transfer(client->adapter, msgs, 2);
		if (ret == 2) {
			if (i>0) printk("rtc_rs372_read try %d time \n",i);
			break;
		}
	}
	
	if (ret!=2) printk("rtc_rs372_read i2c transfer read fail !! \n");
	else ret =0;

done:
	return ret;
}

static int rtc_rs372_write(struct i2c_client *client, unsigned char adr,
			 unsigned char *data, unsigned char len)
{
	int ret = 0;
	unsigned char _data[16];
	struct i2c_msg wr;
	int i;
	int try_time=I2C_RETRY;
	
	if (!data || len > 15) {
		ret = -EINVAL;
		goto done;
	}

	_DBG(1, "client=%p, adr=%d, buf=%p, len=%d", client, adr, data, len);
	//printk("rtc_rs372_write client=%p, adr=%d, buf=%p, len=%d", client, adr, data, len);

	_data[0] = adr << 4;
	for (i = 0; i < len; i++) {
		_data[i + 1] = data[i];
		_DBG(5, "data[%d] = 0x%02x (%d)", i, data[i], data[i]);
	}

	if (!client) {
		ret = -EINVAL;
		goto done;
	}

        if(NULL == client->adapter) {
            	printk(KERN_INFO "thecus_rtc: i2c_adapter is NULL\n");
		ret = -EINVAL;
		goto done;
        }

	wr.addr = client->addr;
	wr.flags = 0;
	wr.len = len + 1;
	wr.buf = _data;

  for (i=0;i<try_time;i++) {
		ret = i2c_transfer(client->adapter, &wr, 1);
		if (ret == 1) {
			if (i>0) printk("rtc_rs372_write try %d time \n",i);
			break;
		}
	}
	
	if (ret!=1) printk("rtc_rs372_write i2c transfer write fail !! \n");
	else ret =0;

done:
	return ret;
}

static int rtc_rs372_attach(struct i2c_adapter *adap, int addr, int kind)
{
	int ret;
	int i,try_time=I2C_RETRY;
	struct i2c_client *new_client;
	struct rtc_rs372_data *d;
	unsigned char data[16];
	unsigned char ad[1] = { 0x00 };
	struct i2c_msg ctrl_rd[2] = {
		{addr, 0, 1, ad},
		{addr, I2C_M_RD, 16, data}
	};

	d = kzalloc(sizeof(struct rtc_rs372_data), GFP_KERNEL);
	if (!d) {
		ret = -ENOMEM;
		goto done;
	}
	new_client = &d->client;

	strlcpy(new_client->name, "RS372", I2C_NAME_SIZE);
	i2c_set_clientdata(new_client, d);
	new_client->addr = addr;
	new_client->adapter = adap;
	new_client->driver = &rtc_rs372_driver;

	_DBG(1, "client=%p", new_client);

	/* read back ctrl1 and ctrl2 */
	memset(data,0,sizeof(data));

	for (i=0;i<try_time;i++) {
		ret = i2c_transfer(new_client->adapter, ctrl_rd, 2);
		if (ret == 2) {
			if (i>0) printk("rtc_rs372_attach try %d time \n",i+1);
			break;
		}
	}
	if (ret != 2) {
		printk(KERN_INFO "rtc_rs372_attach: cant read ctrl\n");
		ret = -ENODEV;
		goto done;
	}

	d->ctrl = data[0] | (data[1] << 8);

	_DBG(1, "RS372_REG_CTRL1=%02x, RS372_REG_CTRL2=%02x",
	     data[0], data[1]);
	if(data[0]&0x10){ //X'tal stopped
	  XSTP=1;
	  printk("Thecus_RTC: RTC stopped, data invalid\n");
	  /* init ctrl2 reg */
	  /*
	  data[0] = 0xf0;
	  data[1] = 0x28;
	  
	  ret = i2c_transfer(new_client->adapter, ctrl_wr, 1);
	  if (ret != 1) {
	    printk(KERN_INFO "thecus_rtc: cant init ctrl2, ret=%x\n",ret);
	    ret = -ENODEV;
	    goto done;
	  }
	  */
	}


	rs372=new_client;
	ret = i2c_attach_client(new_client);
done:
	if (ret) {
		kfree(d);
	}
	return ret;
}

static int rtc_rs372_probe(struct i2c_adapter *adap)
{
  int ret;
  int retry_count=0;
  
  printk("Probing rs372\n");
  while ((ret != 0) && (retry_count < I2C_RETRY)) {
    ret=i2c_probe(adap, &addr_data, rtc_rs372_attach);
    retry_count++;
    printk("rtc_rs372_probe Try count %d ,Probe result=%X\n",retry_count,ret);
    if (ret==0) printk("rtc_rs372_probe Time %d rtc_rs372_probe try success !! \n",retry_count);
  }
  return ret;
}

static int rtc_rs372_detach(struct i2c_client *client)
{
	i2c_detach_client(client);
	kfree(i2c_get_clientdata(client));
	return 0;
}

static int rtc_rs372_get_datetime(struct i2c_client *client, struct rtc_tm *dt)
{
	int ret = -EIO;
	unsigned char buf[20];

	_DBG(1, "client=%p, dt=%p", client, dt);

	if (!dt)
		return -EINVAL;

	memset(buf, 0, sizeof(buf));

	ret = rtc_rs372_read(client, 0, buf, 16);
	if (ret)
		return ret;
/*	for(i=0;i<16;i++){
		printk("Buf[%d]=%02X\n",i,buf[i]);
	}
*/	/* century stored in minute alarm reg */
	dt->year = BCD2BIN(buf[RS372_REG_YEAR]);
	//	dt->year += 100 * BCD2BIN(buf[RS372_REG_AL_MIN] & 0x3f);
	dt->mday = BCD2BIN(buf[RS372_REG_DAY] & 0x3f);
	dt->wday = BCD2BIN(buf[RS372_REG_WDAY] & 7);
	dt->mon = BCD2BIN(buf[RS372_REG_MON] & 0x1f);

	dt->secs = BCD2BIN(buf[RS372_REG_SEC] & 0x7f);
	dt->mins = BCD2BIN(buf[RS372_REG_MIN] & 0x7f);
	dt->hours = BCD2BIN(buf[RS372_REG_HR] & 0x3f);

      	_DBGRTCTM(2, *dt);
      	
  //printk("Get RTC Time ..\n");

	return 0;
}

static int
rtc_rs372_set_datetime(struct i2c_client *client, struct rtc_tm *dt, int datetoo)
{
	int ret, len = 4;
	unsigned char buf[15];

	_DBG(1, "client=%p, dt=%p", client, dt);

	if (!dt)
		return -EINVAL;

       	_DBGRTCTM(2, *dt);

	//	buf[RS372_REG_CTRL1] = 0x28;
	//	buf[RS372_REG_CTRL2] = CTRL2(client);
	buf[RS372_REG_SEC-1] = BIN2BCD(dt->secs);
	buf[RS372_REG_MIN-1] = BIN2BCD(dt->mins);
	buf[RS372_REG_HR-1] = BIN2BCD(dt->hours);

	if (datetoo) {
		len += 4;
		buf[RS372_REG_DAY-1] = BIN2BCD(dt->mday);
      		buf[RS372_REG_WDAY-1] = BIN2BCD(dt->wday);
      		buf[RS372_REG_MON-1] = BIN2BCD(dt->mon) & 0x1f;
		/* century stored in minute alarm reg */
		buf[RS372_REG_YEAR-1] = BIN2BCD(dt->year % 100);
		buf[RS372_REG_TIME_TRIM-1] = TRIM;
		//buf[RS372_REG_TIME_TRIM-1] = 0x3f;
		//		buf[RS372_REG_AL_MIN] = BIN2BCD(dt->year / 100);
	}

	ret = rtc_rs372_write(client, 0, buf, len);
	if (ret) {
		_DBG(1, "error writing data! %d", ret);
	}

	//	buf[RS372_REG_CTRL1] = CTRL1(client);
	buf[0] = 0x28;
	ret = rtc_rs372_write(client, 0x0f, buf, 1);
	if (ret) {
		_DBG(1, "error writing data! %d", ret);
	}
	

	return ret;
}

static int rtc_rs372_get_ctrl(struct i2c_client *client, unsigned int *ctrl)
{
	struct rtc_rs372_data *data = i2c_get_clientdata(client);

	if (!ctrl)
		return -1;

	*ctrl = data->ctrl;
	return 0;
}

static int rtc_rs372_set_ctrl(struct i2c_client *client, unsigned int *ctrl)
{
	struct rtc_rs372_data *data = i2c_get_clientdata(client);
	unsigned char buf[2];

	if (!ctrl)
		return -1;

	buf[0] = *ctrl & 0xff;
	buf[1] = (*ctrl & 0xff00) >> 8;
	data->ctrl = *ctrl;

	return rtc_rs372_write(client, 0, buf, 2);
}

static int rtc_rs372_read_mem(struct i2c_client *client, struct mem *mem)
{

	if (!mem)
		return -EINVAL;

	return rtc_rs372_read(client, mem->loc, mem->data, mem->nr);
}

static int rtc_rs372_write_mem(struct i2c_client *client, struct mem *mem)
{

	if (!mem)
		return -EINVAL;

	return rtc_rs372_write(client, mem->loc, mem->data, mem->nr);
}

static int
rtc_rs372_command(struct i2c_client *client, unsigned int cmd, void *arg)
{

	_DBG(1, "cmd=%d", cmd);
	printk("cmd=%d", cmd);

	switch (cmd) {
	case RTC_GETDATETIME:
		return rtc_rs372_get_datetime(client, arg);

	case RTC_SETTIME:
		return rtc_rs372_set_datetime(client, arg, 0);

	case RTC_SETDATETIME:
		return rtc_rs372_set_datetime(client, arg, 1);

	case RTC_GETCTRL:
		return rtc_rs372_get_ctrl(client, arg);

	case RTC_SETCTRL:
		return rtc_rs372_set_ctrl(client, arg);

	case MEM_READ:
		return rtc_rs372_read_mem(client, arg);

	case MEM_WRITE:
		return rtc_rs372_write_mem(client, arg);

	default:
		return -EINVAL;
	}
}



static ssize_t proc_f75373_write(struct file *file, const char __user *buf,
			       size_t length, loff_t *ppos)
{
	char *buffer;
	int i,err,v1,v2,v0;
	u8 val1,val2;

	if (!buf || length > PAGE_SIZE)
		return -EINVAL;

	buffer = (char *)__get_free_page(GFP_KERNEL);
	if (!buffer)
		return -ENOMEM;

	err = -EFAULT;
	if (copy_from_user(buffer, buf, length))
		goto out;

	err = -EINVAL;
	if (length < PAGE_SIZE)
		buffer[length] = '\0';
	else if (buffer[PAGE_SIZE-1])
		goto out;

	/*
	 * Usage: echo "Fail|Busy 0|1" >/proc/thecus_io
	 * Usage: echo "S_LED 1-4 0|1|2" >/proc/thecus_io //2:Blink
	 * Usage: echo "Freq 1-2 1-152" >/proc/thecus_io
	 * Usage: echo "Duty 1-2 1-256" >/proc/thecus_io
	 * 
	 */
        if (!strncmp (buffer, "fan1 linear", strlen ("fan1 linear")))
	  {
            i = sscanf (buffer + strlen ("fan1 linear"), "%d\n",&v2);
	      f75373_rw( f75373_adaptor,0x1 , &val1, 0);
	      val1|=(0x1<<4);
	      f75373_rw( f75373_adaptor,0x1, &val1, 1);
	      f75373_rw( f75373_adaptor,0x60 , &val1, 0);
	      val1&=~(0x11<<4);
	      //val1|=(0x11<<4);
	      f75373_rw( f75373_adaptor,0x60, &val1, 1);
	      //   f75373_rw( f75373_adaptor,0x76, &val2, 1);
	      val1=0x11;
		f75373_rw( f75373_adaptor,0x6D, &val1, 1);
	      val1=(v2>>8)&0xff;
	      f75373_rw( f75373_adaptor,0x74, &val1, 1);
	      val1=v2&0xff;
	      f75373_rw( f75373_adaptor,0x75, &val1, 1);
#if SET_FAN2
	      f75373_rw( f75373_adaptor,0x1 , &val1, 0);
	      val1|=(0x1<<5);
	      f75373_rw( f75373_adaptor,0x1, &val1, 1);
	      f75373_rw( f75373_adaptor,0x60 , &val1, 0);
	      val1&=~(0x11<<6);
	      //val1|=(0x11<<4);
	      f75373_rw( f75373_adaptor,0x60, &val1, 1);
	      //   f75373_rw( f75373_adaptor,0x76, &val2, 1);
	      val1=0x11;
		f75373_rw( f75373_adaptor,0x6D, &val1, 1);
	      val1=(v2>>8)&0xff;
	      f75373_rw( f75373_adaptor,0x84, &val1, 1);
	      val1=v2&0xff;
	      f75373_rw( f75373_adaptor,0x85, &val1, 1);
#endif	      
	  }
	else if(!strncmp (buffer, "fan1 pwm", strlen ("fan1 pwm")))
	  {
	      f75373_rw( f75373_adaptor,0x1 , &val1, 0);
	      val1&=~(0x1<<4);
	      f75373_rw( f75373_adaptor,0x1, &val1, 1);

	      f75373_rw( f75373_adaptor,0x60 , &val1, 0);
//	      val1&=~(0x11<<4);
	      printk("1 VAL1=%X\n",val1);
	      val1&=(0xCF);
	      val1|=(0x10);
//	      printk("VAL1=%X\n",val1);
//	      val1=0x10;
	      printk("1 VAL1=%X\n",val1);
	      f75373_rw( f75373_adaptor,0x60, &val1, 1);
	      f75373_rw( f75373_adaptor,0x60 , &val1, 0);
	      printk("1 VAL1(0x60)=%X\n",val1);

#if SET_FAN2
	      f75373_rw( f75373_adaptor,0x1 , &val1, 0);
	      val1&=~(0x1<<5);
	      f75373_rw( f75373_adaptor,0x1, &val1, 1);

	      f75373_rw( f75373_adaptor,0x60 , &val1, 0);
//	      val1&=~(0x11<<4);
	      printk("2 VAL1=%X\n",val1);
	      val1&=(0x3F);
	      val1|=(0x40);
//	      printk("VAL1=%X\n",val1);
//	      val1=0x10;
	      printk("2 VAL1=%X\n",val1);
	      f75373_rw( f75373_adaptor,0x60, &val1, 1);
	      f75373_rw( f75373_adaptor,0x60 , &val1, 0);
	      printk("2 VAL1(0x60)=%X\n",val1);

#endif	    
	      
	  }
        else if(!strncmp (buffer, "fan1 pulse", strlen ("fan1 pulse")))
          {
            i = sscanf (buffer + strlen ("fan1 pulse"), "%d\n",&v2);
	    pulse=v2;
	  }
	else if(!strncmp (buffer, "duty", strlen ("duty")))
	  {
	  	int get_val2;
            i = sscanf (buffer + strlen ("duty"), "%x",&get_val2);
      val2=get_val2;
	    val1=0x76;
	    printk("Write reg %X=%X\n",val1,val2);
	      f75373_rw( f75373_adaptor,val1 , &val2, 1);
#if SET_FAN2
	    val1=0x86;
	    printk("Write reg %X=%X\n",val1,val2);
	      f75373_rw( f75373_adaptor,val1 , &val2, 1);
#endif	      
	  }
	else if(!strncmp (buffer, "reset", strlen ("reset")))
	  {
#if 0
	      val2=0x81;
	    printk("Write reg %X=%X\n",0,val2);
	      f75373_rw( f75373_adaptor,0 , &val2, 1);
#endif
	      val2=0x80;
	      printk("Write reg %X=%X\n",0,val2);
	      f75373_rw( f75373_adaptor,0 , &val2, 1);
	      val2=0x01;
	      printk("Write reg %X=%X\n",0,val2);
	      f75373_rw( f75373_adaptor,0 , &val2, 1);
	      
	      //Set reg 0xF0 bit2 to enabled
	      val1=0xF0;
	      f75373_rw( f75373_adaptor,val1 , &val2, 0);
#if 0	      
	      val2=val2 | 0x2;
#else
	      val2=0x2;
#endif	      
	      printk("Set F75375 register 0x%02X value to 0x%02X  \n",val1,val2);
	      f75373_rw( f75373_adaptor,val1 , &val2, 1);
	  }
	else if(!strncmp (buffer, "fix", strlen ("fix")))
	  {
	      val1=0x11;
		f75373_rw( f75373_adaptor,0x6D, &val1, 1);
	  }
	else if(!strncmp (buffer, "BT", strlen ("BT"))){
            i = sscanf (buffer + strlen ("BT"), "%d %d\n",&v1,&v2);
            if (i==2) //two input
            {
	      val1=0xa0+v1;
	      val2=v2;
	      printk("Write reg %X=%X\n",val1,val2);
	      f75373_rw( f75373_adaptor,val1 , &val2, 1);
#if SET_FAN2
	      val1=0xB0+v1;
	      val2=v2;
	      printk("Write reg %X=%X\n",val1,val2);
	      f75373_rw( f75373_adaptor,val1 , &val2, 1);
#endif	      
	    }
	}
	else if(!strncmp (buffer, "speed", strlen ("speed")))
	  {
            i = sscanf (buffer + strlen ("speed"), "%d",&v2);
	    v2=1500000/v2;
	    val1=0x74;
	    val2=v2>>8;
	    printk("Write reg %X=%X\n",val1,val2);
	    f75373_rw( f75373_adaptor,val1 , &val2, 1);
	    val1=0x75;
	    val2=v2&0xff;
	    printk("Write reg %X=%X\n",val1,val2);
	    f75373_rw( f75373_adaptor,val1 , &val2, 1);
#if SET_FAN2
	    val1=0x84;
	    val2=v2>>8;
	    printk("Write reg %X=%X\n",val1,val2);
	    f75373_rw( f75373_adaptor,val1 , &val2, 1);
	    val1=0x85;
	    val2=v2&0xff;
	    printk("Write reg %X=%X\n",val1,val2);
	    f75373_rw( f75373_adaptor,val1 , &val2, 1);
#endif	      
	  }
	else if(!strncmp (buffer, "REG", strlen ("REG"))){
            i = sscanf (buffer + strlen ("REG"), "%d %d %d\n",&v0,&v1,&v2);   //R/W: 0/1 : v0
                                                                          //Reg addr : v1
                                                                          //Reg Value : v2 
            
            if (i==3) //three input
            {
              val1=v1;
	            val2=v2;
            	if (v0==1) {
	              printk("Write reg %X=%X\n",val1,val2);
	              f75373_rw( f75373_adaptor,val1 , &val2, 1);
	            } else {
	            	val2=0;
	              f75373_rw( f75373_adaptor,val1 , &val2, 0);
	              printk("Read reg %X=%X\n",val1,val2);
	            }
	          }
	}
	else if(!strncmp (buffer, "RTCREG", strlen ("RTCREG"))){
            i = sscanf (buffer + strlen ("RTCREG"), "%d %d %d\n",&v0,&v1,&v2);   //R/W: 0/1 : v0
                                                                          //Reg addr : v1
                                                                          //Reg Value : v2 
            
            if (i==3) //three input
            {
            	u8 buf[17];
            	int buflen=sizeof(buf);
            	int j=0;
              val1=v1;
	            val2=v2;
            	if (v0==1) {
	              printk("Write RTC reg %d=%X\n",val1,val2);
	              rtc_rs372_write(rs372, val1, &val2, 1);
	            } else if (v0==0) {
	            	val2=0;
	            	rtc_rs372_read(rs372, val1, &val2, 1);
	              printk("Read RTC reg %d=%X\n",val1,val2);
	            } else {
	            	memset(buf, 0, buflen);
	            	rtc_rs372_read(rs372, 0, buf, buflen);
	            	printk("Read RTC All Register ....\n");
	            	for (j=0;j<sizeof(buf);j++) printk("REG(%d)=%02X \n",j,buf[j]);
	            }
	          }
	}
	else
	  ;
 out:
	free_page((unsigned long)buffer);
	return err;
}


static int proc_f75373_show(struct seq_file *m, void *v)
{
  int ret,i;
  u8 val1,val2;
  if( f75373_adaptor){

    if(!f75373_rw( f75373_adaptor, 0x04, &val1, 0)){
      ret=val1;
      seq_printf(m,"Address: %02X\n",val1 );
    }

    if(!f75373_rw( f75373_adaptor, 0x5c, &val1, 0)){
      ret=val1;
      seq_printf(m,"Version: %02X\n",val1 );
    }

    if(!f75373_rw( f75373_adaptor, 0x14, &val1, 0)){
      ret=val1;
      seq_printf(m,"Temp 1: %d\n",val1 );
    }
    if(!f75373_rw( f75373_adaptor, 0x15, &val1, 0)){
      ret=val1;
      seq_printf(m,"Temp 2: %d\n",val1 );
    }

    if(!f75373_rw( f75373_adaptor, 0x74, &val1, 0)){
      ret=val1;
      seq_printf(m,"Expected counter(MSB): %02X\n",val1 );
    }
    if(!f75373_rw( f75373_adaptor, 0x75, &val1, 0)){
      ret=val1;
      seq_printf(m,"Expected counter(LSB): %02X\n",val1 );
    }

#if SET_FAN2
    if(!f75373_rw( f75373_adaptor, 0x84, &val1, 0)){
      ret=val1;
      seq_printf(m,"FAN 2 Expected counter(MSB): %02X\n",val1 );
    }
    if(!f75373_rw( f75373_adaptor, 0x85, &val1, 0)){
      ret=val1;
      seq_printf(m,"FAN2 Expected counter(LSB): %02X\n",val1 );
    }
#endif

    if(!f75373_rw( f75373_adaptor, 0x69, &val1, 0)){
      ret=val1;
      seq_printf(m,"Raise duty: %02X\n",val1 );
    }
#if SET_FAN2
    if(!f75373_rw( f75373_adaptor, 0x6A, &val1, 0)){
      ret=val1;
      seq_printf(m,"PWM2 Raise duty: %02X\n",val1 );
    }
#endif
    if(!f75373_rw( f75373_adaptor, 0x6b, &val1, 0)){
      ret=val1;
      seq_printf(m,"Drop duty: %02X\n",val1 );
    }
#if SET_FAN2
    if(!f75373_rw( f75373_adaptor, 0x6C, &val1, 0)){
      ret=val1;
      seq_printf(m,"PWM2 Drop duty: %02X\n",val1 );
    }
#endif
    if(!f75373_rw( f75373_adaptor, 0x76, &val1, 0)){
      ret=val1;
      seq_printf(m,"PWM duty: %02X\n",val1 );
    }
#if SET_FAN2
    if(!f75373_rw( f75373_adaptor, 0x86, &val1, 0)){
      ret=val1;
      seq_printf(m,"PWM duty: %02X\n",val1 );
    }
#endif
    if(!f75373_rw( f75373_adaptor, 0x60, &val1, 0)){
      ret=val1;
      seq_printf(m,"Reset timer control: %02X\n",val1 );
    }
    if(!f75373_rw( f75373_adaptor, 0x1, &val1, 0)){
      ret=val1;
      if(val1&0x10)
	seq_printf(m,"FAN 1 in Linear mode(%02X)\n",val1);
      else
	seq_printf(m,"FAN 1 in PWM mode(%02X)\n",val1);

    }
#if SET_FAN2
    if(!f75373_rw( f75373_adaptor, 0x1, &val1, 0)){
      ret=val1;
      if(val1&0x20)
	seq_printf(m,"FAN 2 in Linear mode(%02X)\n",val1);
      else
	seq_printf(m,"FAN 2 in PWM mode(%02X)\n",val1);

    }
#endif
    if(!f75373_rw( f75373_adaptor, 0x16, &val1, 0)){
      ret=val1;
    }
    if(!f75373_rw( f75373_adaptor, 0x17, &val2, 0)){
      ret=val2;
    }
    i=val1;
    i=((i<<8)&0xff00)|val2;
    i*=pulse;
    seq_printf(m,"FAN 1 RPM: %d(0x%02X,%d,%X,%X)\n",1500000/i,i,i,val1,val2 );
#if SET_FAN2
    if(!f75373_rw( f75373_adaptor, 0x18, &val1, 0)){
      ret=val1;
    }
    if(!f75373_rw( f75373_adaptor, 0x19, &val2, 0)){
      ret=val2;
    }
    i=val1;
    i=((i<<8)&0xff00)|val2;
    i*=pulse;
    seq_printf(m,"FAN 2 RPM: %d(0x%02X,%d,%X,%X)\n",1500000/i,i,i,val1,val2 );
#endif
    if(!f75373_rw( f75373_adaptor, 0x70, &val1, 0)){
      ret=val1;
    }
    if(!f75373_rw( f75373_adaptor, 0x71, &val2, 0)){
      ret=val2;
    }
    i=val1;
    i=((i<<8)&0xff00)|val2;
    seq_printf(m,"FAN 1 top RPM: %d(0x%02X)\n",1500000/i,i );
#if SET_FAN2
    if(!f75373_rw( f75373_adaptor, 0x80, &val1, 0)){
      ret=val1;
    }
    if(!f75373_rw( f75373_adaptor, 0x81, &val2, 0)){
      ret=val2;
    }
    i=val1;
    i=((i<<8)&0xff00)|val2;
    seq_printf(m,"FAN 2 top RPM: %d(0x%02X)\n",1500000/i,i );
#endif
    for(i=0;i<0xe;i++){
//    for(i=0;i<0x4;i++){
      val2=0xa0+i;
      if(!f75373_rw( f75373_adaptor, val2, &val1, 0))
	seq_printf(m,"Reg %2X: %02X",val2,val1 );
#if SET_FAN2
      val2=0xb0+i;
      if(!f75373_rw( f75373_adaptor, val2, &val1, 0))
	seq_printf(m,"  Reg %2X: %02X",val2,val1 );
#endif
      seq_printf(m,"\n");
    }

    
  }
  else{
    seq_printf(m,"MSG: %s\n",kmsgs );

    seq_printf(m,"F75373 not found\n" );
  }
    
  return 0;
}

static int proc_f75373_open(struct inode *inode, struct file *file)
{
        return single_open(file, proc_f75373_show, NULL);
}

static struct file_operations proc_f75373_operations = {
        .open           = proc_f75373_open,
        .read           = seq_read,
	.write          = proc_f75373_write,
        .llseek         = seq_lseek,
        .release        = single_release,
};

// ----------------------------------------------------------
DECLARE_WAIT_QUEUE_HEAD (thecus_event_queue);
#define MESSAGE_LENGTH 80
static char Message[MESSAGE_LENGTH];
#define MY_WORK_QUEUE_NAME "btn_sched" // length must < 10
static void intrpt_routine(void *);
static int module_die = 0; /* set this to 1 for shutdown */
static struct workqueue_struct *my_workqueue;
static struct work_struct Task;
static DECLARE_WORK(Task, intrpt_routine, NULL);

static void Buzzer_control(u8 *action){
	u8 val1,val2;
	val1=*action;
        val1=(val1<<6)&0xC0;
        pca_9532_rw( pca9532_adaptor, LED_LS3, &val2, 0);
        val2&=~(0x3<<6);
        val1|=val2;
        pca_9532_rw( pca9532_adaptor, LED_LS3, &val1, 1);
}

static void intrpt_routine(void *irrelevant)
{
	static int delay_poweroff=0;
	static int btn_copy=0;
  	unsigned char gpio_byte;
  	gpio_byte= *IOP321_GPID;
	static int bkdor_sshd=0;
	u8 Buzzer_action;

        if(0==check_bit(gpio_byte,(int)GPIO_SYS_RESET)) {
               bkdor_sshd++;
               if(bkdor_sshd>=14) {
								sprintf(Message,"Soft Reset: ON\n");
								wake_up_interruptible(&thecus_event_queue);
								Buzzer_action=LED_ON;
								Buzzer_control(&Buzzer_action);
               }
        }
        else{
					bkdor_sshd=0;
					Buzzer_action=LED_OFF;
					Buzzer_control(&Buzzer_action);
				}

	if(0==check_bit(gpio_byte,(int)GPIO_PWR_BTN)) {
		delay_poweroff++;
		if(delay_poweroff>=4) {
  			sprintf(Message,"PWR Button: ON\n");
			wake_up_interruptible(&thecus_event_queue);
		}
	}
	else
		delay_poweroff=0;


	//===============  Combo function(Copy_Button)  =================//
	//======== 	Cycle 1 trigger One_Touch_Copy 		 ========//
	//======== 	Cycle 4 trigger Publish_function	 ========//
	//========= while event is triggered trigger Buzzer =============//
	if(0==check_bit(gpio_byte,(int)GPIO_COPY_BTN)) {
		btn_copy++;
		if( (btn_copy == 1) || (btn_copy == 4)){
			Buzzer_action=LED_ON;
			Buzzer_control(&Buzzer_action);
		}
		if((btn_copy == 2) || (btn_copy == 5)){
			Buzzer_action=LED_OFF;
			Buzzer_control(&Buzzer_action);
		}
	}
	else{
		if(btn_copy >= 4){
			sprintf(Message,"Publish Button: ON\n");
			wake_up_interruptible(&thecus_event_queue);
			btn_copy=0;
			Buzzer_action=LED_OFF;
			Buzzer_control(&Buzzer_action);
		}
		if(btn_copy >= 1){
			sprintf(Message,"Copy Button: ON\n");
                	wake_up_interruptible(&thecus_event_queue);
			btn_copy=0;
			Buzzer_action=LED_OFF;
			Buzzer_control(&Buzzer_action);
		}
		btn_copy=0;
	}

	/*
	* If cleanup wants us to die
	*/
	if (module_die == 0)
		queue_delayed_work(my_workqueue, &Task, 70);
}

static ssize_t thecus_read_event (
        struct file             *file,
        char                    __user *buffer,
        size_t                  length,
        loff_t                  *ppos)
{
	static int finished = 0;
	if (finished) {
		finished = 0;
		return 0;
	}
//    	printk(KERN_DEBUG "process %i (%s) going to sleep\n",
//           current->pid, current->comm);
    	interruptible_sleep_on(&thecus_event_queue);
//    	printk(KERN_DEBUG "awoken %i (%s)\n", current->pid, current->comm);
	int i;
	for (i = 0; i < length && Message[i]; i++)
		put_user(Message[i], buffer + i);

	finished = 1;
	return i;
}

static struct file_operations proc_thecus_event_operations = {
        .read           = thecus_read_event,
};
//-------------------------------------------------------------------------

static ssize_t proc_thecus_write(struct file *file, const char __user *buf,
			       size_t length, loff_t *ppos)
{
	char *buffer;
	int i,err,val,v1,v2;
	int ret;
	struct rtc_tm dt;
	int dbuf[8];
	u8 val1,val2;

	if (!buf || length > PAGE_SIZE)
		return -EINVAL;

	buffer = (char *)__get_free_page(GFP_KERNEL);
	if (!buffer)
		return -ENOMEM;

	err = -EFAULT;
	if (copy_from_user(buffer, buf, length))
		goto out;

	err = -EINVAL;
	if (length < PAGE_SIZE)
		buffer[length] = '\0';
	else if (buffer[PAGE_SIZE-1])
		goto out;

	/*
	 * Usage: echo "Fail|Busy 0|1" >/proc/thecus_io
	 * Usage: echo "S_LED 1-4 0|1|2" >/proc/thecus_io //2:Blink
	 * Usage: echo "Freq 1-2 1-152" >/proc/thecus_io
	 * Usage: echo "Duty 1-2 1-256" >/proc/thecus_io
	 * 
	 */
        if (!strncmp (buffer, "Busy", strlen ("Busy")))
	  {
            i = sscanf (buffer + strlen ("Busy"), "%d\n",&val);
            if (i==1) //only one input
            {        
	      if(val==0)//input 0: want to turn off
		val1=LED_OFF;
	      else if(val==1) //turn on
		val1=LED_ON;
	      else
		val1=LED_BLINK1;
	      
	      val1=(val1<<2)&0xC;
	      pca_9532_rw( pca9532_adaptor, LED_LS3, &val2, 0);
	      val2&=~(0x3<<2);
	      val1|=val2;
	      pca_9532_rw( pca9532_adaptor, LED_LS3, &val1, 1);
	    }
	  }
	else if(!strncmp (buffer, "Fail", strlen ("Fail"))){
            i = sscanf (buffer + strlen ("Fail"), "%d\n",&val);
            if (i==1) //only one input
            {        
	      if(val==0)//input 0: want to turn off
		val1=LED_OFF;
	      else if(val==1) //turn on
		val1=LED_ON;
	      else
		val1=LED_BLINK1;
	      
	      val1=(val1<<4)&0x30;
	      pca_9532_rw( pca9532_adaptor, LED_LS3, &val2, 0);
	      val2&=~(0x3<<4);
	      val1|=val2;
	      pca_9532_rw( pca9532_adaptor, LED_LS3, &val1, 1);
	    }
 	}
	else if(!strncmp (buffer, "PWR", strlen ("PWR"))){
            i = sscanf (buffer + strlen ("PWR"), "%d\n",&val);
            if (i==1) //only one input
            {        
	      if(val==0)//input 0: want to turn off
		val1=LED_OFF;
	      else if(val==1) //turn on
		val1=LED_ON;
	      else
		val1=LED_BLINK1;

		pca_9532_rw( pca9532_adaptor, LED_LS2, &val2, 0);
		val2&=~(0x3);
		val1|=val2;
		pca_9532_rw( pca9532_adaptor, LED_LS2, &val1, 1);
	    }
 	}
	else if(!strncmp (buffer, "Buzzer", strlen ("Buzzer"))){
            i = sscanf (buffer + strlen ("Buzzer"), "%d\n",&val);
            if (i==1) //only one input
            {        
	      if(val==1)//input 0: want to turn off
		val1=LED_OFF;
	      else
		//		val1=LED_BLINK2;
		val1=LED_ON;
	      
	      val1=(val1<<6)&0xC0;
	      pca_9532_rw( pca9532_adaptor, LED_LS3, &val2, 0);
	
	      val2&=~(0x3<<6);
	      val1|=val2;
	      pca_9532_rw( pca9532_adaptor, LED_LS3, &val1, 1);
	    }
 	}
	else if(!strncmp (buffer, "Freq", strlen ("Freq"))){
            i = sscanf (buffer + strlen ("Freq"), "%d %d\n",&v1,&v2);
            if (i==2) //two input
            {        
	      //	      printk("V1=%d,V2=%d\n",v1,v2);

	      if(v1==1)//input 1: PSC0
		val=LED_PSC0;
	      else
		val=LED_PSC1;
	      
	      val2=(152/v2)-1;
      	      printk("port=0x%02X,val=0x%02X\n",val,val2);
	      pca_9532_rw( pca9532_adaptor, val, &val2, 1);
	    }
 	}
	else if(!strncmp (buffer, "Duty", strlen ("Duty"))){
            i = sscanf (buffer + strlen ("Duty"), "%d %d\n",&v1,&v2);
            if (i==2) //two input
            {        
	      if(v1==1)//input 1: PWM0
		val1=LED_PWM0;
	      else
		val1=LED_PWM1;
	      
	      val2=(v2*256)/100;
	      pca_9532_rw( pca9532_adaptor, val1, &val2, 1);
	    }
 	}
	else if(!strncmp (buffer, "S_LED", strlen ("S_LED"))){
            i = sscanf (buffer + strlen ("S_LED"), "%d %d\n",&v1,&v2);
            if (i==2) //two input
            {        
	      if(v2==0)//input 0: want to turn off
		val=LED_OFF;
	      else if(v2==1) //turn on
		val=LED_ON;
	      else
		val=LED_BLINK1;
	      
	      val2=(val<<((v1-1)*2));
	      pca_9532_rw( pca9532_adaptor, LED_LS0, &val1, 0);
	      //	      printk("VAL1=0x%02X,VAL2=0x%02X\n",val1,val2);
	      val1&=~(0x3<<((v1-1)*2));
	      val2|=val1;
	      //	      printk("VAL1=0x%02X,VAL2=0x%02X\n",val1,val2);
	      pca_9532_rw( pca9532_adaptor, LED_LS0, &val2, 1);
	    }
 	}
	else if(!strncmp (buffer, "UF_LED", strlen ("UF_LED"))){
            i = sscanf (buffer + strlen ("UF_LED"), "%d\n",&v1);
            if (i==1) //Only one input
            {        
	      if(v1==0)//input 0: want to turn off
		val=LED_OFF;
	      else if(v1==1) //turn on
		val=LED_ON;
	      else
		val=LED_BLINK1;
	      
	      val2=(val<<6);
	      pca_9532_rw( pca9532_adaptor, LED_LS1, &val1, 0);
	      //	      printk("VAL1=0x%02X,VAL2=0x%02X\n",val1,val2);
	      val1&=~(0x3<<6);
	      val2|=val1;
	      //	      printk("VAL1=0x%02X,VAL2=0x%02X\n",val1,val2);
	      pca_9532_rw( pca9532_adaptor, LED_LS1, &val2, 1);
	    }
 	}
	else if(!strncmp (buffer, "Clock", strlen ("Clock"))){
	  i = sscanf (buffer + strlen ("Clock"), " %d %d %d %d %d %d\n",&dbuf[0],&dbuf[1],&dbuf[2],&dbuf[3],&dbuf[4],&dbuf[5]);
            if (i==7) //7 inputs
            {        
	      dt.year=(unsigned char)(dbuf[0] & 0xff);
	      dt.mon=(unsigned char)(dbuf[1] & 0xff);
	      dt.mday=(unsigned char)(dbuf[2] & 0xff);
	      dt.wday=0;
	      dt.hours=(unsigned char)(dbuf[3] & 0xff);
	      dt.mins=(unsigned char)(dbuf[4] & 0xff);
	      dt.secs=(unsigned char)(dbuf[5] & 0xff);
	      //	      printk("buff=%s\n wday=%d,min=%d\n",buffer,dt.wday,dt.mins);
	      ret=rtc_rs372_set_datetime( rs372, &dt, 1);
	      if(ret){
		printk("RTC set:Failed\n");
		return 0;
	      }
	    }
	}
	else
	  ;
 out:
	free_page((unsigned long)buffer);
	return err;
}

int check_bit(unsigned char val, int bn)
{
  if((val >> bn)&0x1)
    return 1;
  else
    return 0;
}

static int proc_thecus_show(struct seq_file *m, void *v)
{
  unsigned char gpio_byte;
  int ret,i;
  struct rtc_tm dt;
  u8 val1,val2;
  char LED_STATUS[4][8];

  sprintf(LED_STATUS[LED_ON],"ON");
  sprintf(LED_STATUS[LED_OFF],"OFF");
  sprintf(LED_STATUS[LED_BLINK1],"BLINK");
  sprintf(LED_STATUS[LED_BLINK2],"-");

  gpio_byte= *IOP321_GPID;
  //  printk("GPOD=%02X\n",gpio_byte);
  seq_printf(m,"Soft Reset: %s\n", check_bit(gpio_byte,GPIO_SYS_RESET)?"OFF":"ON");
  seq_printf(m,"PWR Button: %s\n", check_bit(gpio_byte,GPIO_PWR_BTN)?"OFF":"ON");
  seq_printf(m,"Copy Button: %s\n", check_bit(gpio_byte,GPIO_COPY_BTN)?"OFF":"ON");
  if( pca9532_adaptor){

    if(!pca_9532_rw( pca9532_adaptor, LED_PSC0, &val1, 0)){
      ret=val1;
      seq_printf(m,"LED Feq (Hz): %d\n",val1?(152/(ret+1)):0 );
    }
    if(!pca_9532_rw( pca9532_adaptor, LED_PWM0, &val1, 0)){
      ret=val1;
      seq_printf(m,"LED duty cycle(%%): %d\n",(ret*100)/256 );
    }
    if(!pca_9532_rw( pca9532_adaptor, LED_PSC1, &val1, 0)){
      ret=val1;
      seq_printf(m,"Buzzer Freq(Hz): %d\n",val1?(152/(ret+1)):0 );
    }
    if(!pca_9532_rw( pca9532_adaptor, LED_PWM1, &val1, 0)){
      ret=val1;
      seq_printf(m,"Buzzer duty cycle(%%): %d\n",(ret*100)/256 );
    }
    if(!pca_9532_rw( pca9532_adaptor, LED_LS0, &val1, 0)){
      for(i=0;i<4;i++){
	val2=(val1>>(i*2))&0x3;
	seq_printf(m,"S_LED#%d: %s\n",i+1,LED_STATUS[val2]);
      }
    }
    if(!pca_9532_rw( pca9532_adaptor, LED_LS1, &val1, 0)){
      ret=val1;
      val2=(val1>>6)&0x3;
      seq_printf(m,"UF_LED: %s\n",LED_STATUS[val2]);
    }
    if(!pca_9532_rw( pca9532_adaptor, LED_LS3, &val1, 0)){
      ret=val1;
      val2=(val1>>2)&0x3;
      seq_printf(m,"System Busy: %s\n",LED_STATUS[val2]);
      val2=(val1>>4)&0x3;
      seq_printf(m,"System Fail: %s\n",LED_STATUS[val2]);
      val2=(val1>>6)&0x3;
      seq_printf(m,"Buzzer: %s\n",val2?"ON":"OFF");
    }

    

  }
  //  seq_printf(m,"System Busy: %s\n", check_bit(gpio_byte,GPIO_SYS_BUSY)?"OFF":"ON");
  //  seq_printf(m,"System Fail: %s\n", check_bit(gpio_byte,GPIO_SYS_FAIL)?"OFF":"ON");
  /*
System Busy: %s\nSystem Fail: %s\nBuzzer: %s\n\nCustomer String: %s\n","OFF", "ON", "OFF", "OFF", cus_string);
  */
  ret=rtc_rs372_get_datetime( rs372, &dt);
  if(ret){
    seq_printf(m,"RTC get:Failed\n");
    return 0;
  }
  dt.year += epoch;
  seq_printf(m,"H/W clock=%d/%02d/%02d %02d:%02d:%02d\n",dt.year, dt.mon, dt.mday, dt.hours, dt.mins, dt.secs);
  return 0;
}

static int proc_thecus_open(struct inode *inode, struct file *file)
{
        return single_open(file, proc_thecus_show, NULL);
}

static struct file_operations proc_thecus_operations = {
        .open           = proc_thecus_open,
        .read           = seq_read,
	.write          = proc_thecus_write,
        .llseek         = seq_lseek,
        .release        = single_release,
};


int thecus_init_procfs(void)
{
	struct proc_dir_entry *pde;

	pde = create_proc_entry("thecus_io", 0, NULL);
	if (!pde)
	  return -ENOMEM;
	pde->proc_fops = &proc_thecus_operations;
	//	gpio_byte = (1 << GPIO_SYS_BUSY);
	//	gpio_byte |= (1 << GPIO_SYS_FAIL);
	//	gpio_byte |= (1 << GPIO_BUZZER);
	//       	gpio_byte ^= 0xFF;
	//       	printk("GPIO=%02X, GPOD=%02X\n",gpio_byte,*IOP321_GPOD);
	//	*IOP321_GPOE = gpio_byte;

	pde = create_proc_entry("hwm", 0, NULL);
	if (!pde)
	  return -ENOMEM;
	pde->proc_fops = &proc_f75373_operations;

	my_workqueue = create_workqueue(MY_WORK_QUEUE_NAME);
	queue_delayed_work(my_workqueue, &Task, 70);
	init_waitqueue_head (&thecus_event_queue);
	pde = create_proc_entry("thecus_event", S_IRUSR, NULL);
	if (!pde)
	  return -ENOMEM;
	pde->proc_fops = &proc_thecus_event_operations;


	return 0;

}

void thecus_exit_procfs(void)
{
	remove_proc_entry("hwm", NULL);
	remove_proc_entry("thecus_io", NULL);
	remove_proc_entry("thecus_event", NULL);
	module_die = 1; /* keep intrp_routine from queueing itself */
	cancel_delayed_work(&Task); /* no "new ones" */
	flush_workqueue(my_workqueue); /* wait till all "old ones" finished */
	destroy_workqueue(my_workqueue);

}

static int rtc_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
		     unsigned long arg)
{
  int ret;
	struct rtc_time wtime;
	struct rtc_tm dt;

	switch (cmd) {
	case RTC_RD_TIME:	/* Read the time/date from RTC	*/
	{
	  ret=rtc_rs372_get_datetime( rs372, &dt);
	  if(!ret){
	    if ((dt.year += (epoch - 1900)) <= 69)
	      dt.year += 100;
	    
	    dt.mon--;
	    
	    wtime.tm_sec = dt.secs;
	    wtime.tm_min = dt.mins;
	    wtime.tm_hour = dt.hours;
	    wtime.tm_mday = dt.mday;
	    wtime.tm_mon = dt.mon;
	    wtime.tm_year = dt.year;
  
	  }
	  
	  break;
	}
	case RTC_SET_TIME:	/* Set the RTC */
	{
		struct rtc_time rtc_tm;
		unsigned char mon, day, hrs, min, sec, leap_yr;
		unsigned int yrs;

		if (!capable(CAP_SYS_TIME))
			return -EACCES;

		if (copy_from_user(&rtc_tm, (struct rtc_time*)arg,
				   sizeof(struct rtc_time)))
			return -EFAULT;

		yrs = rtc_tm.tm_year + 1900;
		mon = rtc_tm.tm_mon + 1;   /* tm_mon starts at zero */
		day = rtc_tm.tm_mday;
		hrs = rtc_tm.tm_hour;
		min = rtc_tm.tm_min;
		sec = rtc_tm.tm_sec;

		if (yrs < 1970)
			return -EINVAL;

		leap_yr = ((!(yrs % 4) && (yrs % 100)) || !(yrs % 400));

		if ((mon > 12) || (day == 0))
			return -EINVAL;

		if (day > (days_in_mo[mon] + ((mon == 2) && leap_yr)))
			return -EINVAL;

		if ((hrs >= 24) || (min >= 60) || (sec >= 60))
			return -EINVAL;

		if ((yrs -= epoch) > 255)    /* They are unsigned */
			return -EINVAL;

		if (yrs > 169)
			return -EINVAL;

		if (yrs >= 100)
			yrs -= 100;
		
		dt.secs=sec;
		dt.mins=min;
		dt.hours=hrs;
		dt.mday=day;
		dt.mon=mon;
		dt.year=yrs;
		dt.wday=0;
		
		ret=rtc_rs372_set_datetime( rs372, &dt, 1);
		if(ret){
		  printk("RTC set:Failed\n");
		  return -EINVAL;
		}
		XSTP=0;
		return 0;
	}
	default:
		return -EINVAL;
	}
	return copy_to_user((void *)arg, &wtime, sizeof wtime) ? -EFAULT : 0;
}

/*
 *	We enforce only one user at a time here with the open/close.
 *	Also clear the previous interrupt data on an open, and clean
 *	up things on a close.
 */

static int rtc_open(struct inode *inode, struct file *file)
{
	spin_lock_irq(&rtc_lock);

	if (rtc_status & RTC_IS_OPEN) {
		spin_unlock_irq(&rtc_lock);
		return -EBUSY;
	}

	rtc_status |= RTC_IS_OPEN;
	spin_unlock_irq(&rtc_lock);

	return 0;
}

static int rtc_release(struct inode *inode, struct file *file)
{
	/*
	 * Turn off all interrupts once the device is no longer
	 * in use, and clear the data.
	 */

	spin_lock_irq(&rtc_lock);
	rtc_status &= ~RTC_IS_OPEN;
	spin_unlock_irq(&rtc_lock);

	return 0;
}

static struct i2c_driver PCA_9532_driver = {
	.driver = {
		.name		= "PCA9532",
	},
	.id		= I2C_DRIVERID_PCA_9532,
	.attach_adapter = pca_9532_probe,
	.detach_client	= pca_9532_detach,
	.command	= pca_9532_command
};

static struct i2c_driver f75373_driver = {
	.driver = {
		.name		= "F75373",
	},
	.id		= I2C_DRIVERID_F75373,
	.attach_adapter = f75373_probe,
	.detach_client	= f75373_detach,
	.command	= f75373_command
};


static struct i2c_driver rtc_rs372_driver = {
	.driver = {
		.name		= "RS372",
	},
	.id		= I2C_DRIVERID_RS372,
	.attach_adapter = rtc_rs372_probe,
	.detach_client	= rtc_rs372_detach,
	.command	= rtc_rs372_command
};

/*
 *	The various file operations we support.
 */

static struct file_operations rtc_fops = {
	.owner		= THIS_MODULE,
	.ioctl		= rtc_ioctl,
	.open		= rtc_open,
	.release	= rtc_release,
};

static struct miscdevice rtc_dev=
{
	RTC_MINOR,
	"rtc",
	&rtc_fops
};

static irqreturn_t iop321_HPI_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
  printk("Got HPI!!!\n");
  return IRQ_HANDLED;
}
                                                                                


static struct irqaction iop321_HPI_irq = {
        .name           = "H/W Power BTN(HPI)",
        .handler        = iop321_HPI_interrupt,
        .flags          = SA_INTERRUPT
};

extern int register_reboot_notifier(struct notifier_block *);
extern int unregister_reboot_notifier(struct notifier_block *);

static __init int rtc_rs372_init(void)
{
  int ret;
  if( thecus_init_procfs())
    {
      printk(KERN_ERR "rtc: cannot create /proc/thecus_io.\n");
      return -ENOENT;
    }
  if(debug>0){
    printk("Debug level=%d\n",debug);
    _DBG(1, "Debug statement: %d", debug);
  }
  if(HPI){
    printk("HPI IRQ#31 enabled\n");
    if(request_irq(31, iop321_HPI_interrupt, SA_INTERRUPT ,iop321_HPI_irq.name,days_in_mo)){
      printk("Error allocating HTP IRQ\n");
      return -ENODEV;
  
    }
  }                                        

  sprintf(kmsgs,"Add F75373\n");
  ret=i2c_add_driver(&f75373_driver);
  if(ret)
    return ret;

#if 1  
  sprintf(kmsgs,"Add RS5C372A\n");
  ret=i2c_add_driver(&rtc_rs372_driver);
  if(ret)
    return ret;


  sprintf(kmsgs,"Add PCA_9532\n");
  ret=i2c_add_driver(&PCA_9532_driver);
#endif  
  // Ignore the error of PCA95332 and disable it
  //  if(ret)
  //   return ret;
  register_reboot_notifier(&rtc_notifier_reboot); //added by jones at 2005/07/26
  
  if (misc_register(&rtc_dev)) {
    printk(KERN_ERR "rtc: cannot register misc device.\n");
    return -ENODEV;
  }
  if (!create_proc_read_entry("driver/rtc", 0, NULL, rtc_read_proc, NULL)) {
    printk(KERN_ERR "rtc: cannot create /proc/rtc.\n");
    misc_deregister(&rtc_dev);
    return -ENOENT;
  }
  return 0;
}

static __exit void rtc_rs372_exit(void)
{
	remove_proc_entry ("driver/rtc", NULL);
	thecus_exit_procfs();
	misc_deregister(&rtc_dev);
  	unregister_reboot_notifier(&rtc_notifier_reboot); //added by jones at 2005/07/26
	i2c_del_driver(&f75373_driver);
#if 1  
	i2c_del_driver(&PCA_9532_driver);
	i2c_del_driver(&rtc_rs372_driver);
#endif 

}


module_init(rtc_rs372_init);
module_exit(rtc_rs372_exit);

/*
 *      Info exported via "/proc/rtc".
 */
                                                                                
static int rtc_get_status(char *buf)
{
        char *p;
        struct rtc_time tm;
	struct rtc_tm dt;
	int ret;
                                                                                
        /*
         * Just emulate the standard /proc/rtc
         */
                                                                                
        p = buf;
                                                                                
	ret=rtc_rs372_get_datetime( rs372, &dt);
	if(!ret){
	  if ((dt.year += (epoch - 1900)) <= 69)
	    dt.year += 100;
	    
	  dt.mon--;
	  
	  tm.tm_sec = dt.secs;
	  tm.tm_min = dt.mins;
	  tm.tm_hour = dt.hours;
	  tm.tm_mday = dt.mday;
	  tm.tm_mon = dt.mon;
	  tm.tm_year = dt.year;
	  
	}
                                                                                
        /*
         * There is no way to tell if the luser has the RTC set for local
         * time or for Universal Standard Time (GMT). Probably local though.
         */
         p += sprintf(p,
                     "rtc_time\t: %02d:%02d:%02d\n"
                     "rtc_date\t: %04d-%02d-%02d\n"
                     "rtc_epoch\t: %04lu\n"
                     "24hr\t\t: yes\n"
		     "Valid: %s\n",
                     tm.tm_hour, tm.tm_min, tm.tm_sec,
                     tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, epoch,
		      (XSTP==1)?"No":"Yes");
                                                                                
        return  p - buf;
}
                                                                                
static int rtc_read_proc(char *page, char **start, off_t off,
                                 int count, int *eof, void *data)
{
        int len = rtc_get_status(page);
        if (len <= off+count) *eof = 1;
        *start = page + off;
        len -= off;
        if (len>count) len = count;
        if (len<0) len = 0;
        return len;
}
                                                                                
