/*******************************************************************************
  Copyright (c) 2003-2005, Intel Corporation
  All rights reserved.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License version 2 as published
  by the Free Software Foundation. Alternatively, this software may be
  distributed under the terms of the BSD license. Please see the accompanied
  file COPYING for copyright licensing terms.
*******************************************************************************/
#ifndef GD31244_H
#define GD31244_H

/******************************************************************************/
/*                                                                            */
/*                  I N C L U D E   S T A T E M E N T S                       */
/*                                                                            */
/******************************************************************************/

#include <sata_debug.h>
#include <sata_types.h>		/* Common data types used in SATA driver */
#include <sata_adapter.h>	/* SATA Adapter APIs */
#include <sata_port.h>		/* need this for port reset */
#include <sata_device.h>	/* SATA Drive APIs   */
#include <sata_lib.h>		/* MAX_TCQ_DEPTH, MAX_PORTS_PER_CONTROLLER */

#include <linux/types.h>
#include <linux/kdev_t.h>
#include <linux/list.h>		/* for doubly linked lists used in TCQ */

/******************************************************************************/
/*                                                                            */
/*                C O N S T A N T   D E F I N I T I O N S                     */
/*                                                                            */
/******************************************************************************/
#define PCI_DEVICE_ID_INTEL_ARTISEA 0x3200
#define PCI_VENDOR_ID_INTEL         0x8086

#define GD31244_MEMMAP_SZ ( ARTISEA_COMMON_PORT_REG_SIZE + \
                            ARTISEA_PORT_COUNT * ARTISEA_SATA_PORT_REG_SIZE )

#define INVALID_TAG     (-1)
#ifndef NULL_POINTER
#define NULL_POINTER    (0)
#endif

/* The maximum depth of the Tagged Command Queue ( 1 or 32 )*/
//  MAX_TCQ_DEPTH defined in sata_lib.h
#define MAX_COMMANDS        MAX_TCQ_DEPTH	/* Values might differ to simulate TCQ  */
#define GD31244_DEVCANQUEUE MAX_TCQ_DEPTH
#define GD31244_CANQUEUE    (4 * GD31244_DEVCANQUEUE)

#define DDT_ENTRY_BYTES (8)	/* = sizeof(DDT_ENTRY_T) */

/* In LBA 48, you can transfer a maximum of 0xFFFF sectors at a time. */
/* For a sector size of 512, you can effectively transfer 2^25 (32Mb) */
/* of data Maximum in one command. The DMA h/w imposes amax size of   */
/* 2^16 bytes (64K),for each SGL entry. Therefore we need a max of 512*/
/* Entries to support Maximum LBA48 data transfers. This translates to*/
/* 4Kb memory requirement ( = 1 Page) per DDT. With TCQ, and say a    */
/* queue depth of 32, we will need 32 DDTs per drive = 128 Kb of mem  */
/* only for the DDT's */
#define LINUX_DDT_ENTRIES	512	/*(PAGE_SIZE / (2 * DDT_ENTRY_BYTES)) */

#define GD31244_MAX_CMD_LEN 10
#define GD31244_INFO_SZ     256	/* Max Size of info string */

extern const char gd31244_version_str[];
extern struct Scsi_Host *first_shp;

/******************************************************************************/
/*                                                                            */
/*                   M A C R O   D E F I N I T I O N S                        */
/*                                                                            */
/******************************************************************************/

/* Param is of type Scsi_Host*. Returns the HOST_INFO pointer. NULL on errors*/
#define HOST_INFO(host)       ( (HOST_INFO_T *)(  (host) != NULL_POINTER ?    \
                                                  (host)->hostdata : NULL ) )

/* Param is of type Scsi_Device*. Returns the DEVICE_INFO pointer.NULL on errors*/
#define DEVICE_INFO(scsi_dev) ( (DEVICE_INFO_T*)((scsi_dev) != NULL_POINTER ? \
                                                 (scsi_dev)->hostdata : NULL ) )

/* Param is of type struct scsi_cmnd*. Returns the CMD_INFO pointer. NULL on errors*/
#define CMD_INFO(SCpnt)       ( (CMD_INFO_T*)( (SCpnt) != NULL_POINTER ?   \
                                               (SCpnt)->host_scribble : NULL ) )

/* Param is of type Scsi_Device*. Returns the devTag. INVALID_TAG on errors*/
#define DRIVE_TAG(scsi_dev)   ( (DEVICE_INFO(scsi_dev) != NULL_POINTER) ?    \
                                DEVICE_INFO(scsi_dev)->dev_tag : INVALID_TAG )

/* Param is of type Scsi_Device*. Returns the adapterTag. INVALID_TAG on errors*/
/* For Scsi_Host* params, extract the host_info and access the adapter tag from*/
/* there. e.g. HOST_INFO(shp)->adapter_tag                                     */
#define ADAPTER_TAG(scsi_dev) ( (DEVICE_INFO(scsi_dev) != NULL_POINTER) ?    \
                               DEVICE_INFO(scsi_dev)->adapter_tag : INVALID_TAG )

#define PORT_TAG(scsi_dev)    ( (DEVICE_INFO(scsi_dev) != NULL_POINTER) ? \
                               DEVICE_INFO(scsi_dev)->port_tag : INVALID_TAG )

/* Param is of type Scsi_Host*. Returns the pci_dev pointer. NULL on errors*/
#define HOST_PDEV(host)  ( (struct pci_dev *)( (host) != NULL_POINTER ?    \
                                                  ((HOST_INFO_T *)(struct Scsi_Host *)host->hostdata)->pdev : NULL ))

/* Param is of type struct scsi_device *. Returns the pci_dev pointer. NULL on errors*/
#define DEVICE_PDEV(scsi_dev) ( (scsi_dev) != NULL_POINTER ? \
                                  (HOST_PDEV((struct scsi_device *)(scsi_dev)->host)): NULL )
#ifndef UPPER_32_BITS
#define UPPER_32_BITS(addr)   (( (addr) >> 32) & 0xFFFFFFFF)
#endif

#ifndef LOWER_32_BITS
#define LOWER_32_BITS(addr)   (  (addr)        & 0xFFFFFFFF)
#endif

/******************************************************************************/
/*                                                                            */
/*                    T Y P E   D E F I N I T I O N S                         */
/*                                                                            */
/******************************************************************************/

//typedef struct list_head list_t;

/******************************************************************************/
/*                                                                            */
/*    V A R I A B L E   A N D   S T R U C T U R E   D E F I N I T I O N S     */
/*                                                                            */
/******************************************************************************/

#define SATA_CMD_FREE       0x0
#define SATA_CMD_PENDING    0x1
#define SATA_CMD_ACTIVE     0x2

/* The cmd_info_s*/
typedef struct cmd_info_s {
/** Private Data members - Should NOT be set directly, but these can be    **/
/** read anywhere in the code after they are initialized                   **/

	struct list_head list;	/* prev/next pointers for linked list */
	/* This HAS to be the FIRST element in the structure */

	/* slot_no is the index # into commands array... It is also the index into */
	/* the ddt array. The value of slot_no and ddt_ptr is initialized only once */
	/* at start up. The slot # is used as the TAG for TCQ */
	int slot_no;

	/* During initialization, the ddt_ptr is initialized to                  */
	/* &device_info.ddt[slot_no] This value is never changed after that. The */
	/* DDT should be accessed only Via this pointer. (except during          */
	/* initialization and cleanup)                                           */
	DDT_T *ddt_ptr;

/** Public Data members **/

	struct scsi_cmnd *SCpnt;	/* Pointer to the parent struct scsi_cmnd            */
	unsigned int start_sector;	/* Starting sector # for the transfer (LBA32) */
	unsigned int nr_sectors;	/* Number of sectors to transfer.             */
	int sata_cmd;		/* The SATA command opcode                    */
	int cmd_state;		/* free = 0, pending = 1, active = 2 */
	int sg_dma_direction;	/* SG DMA mapping direction */
	int sg_nents;		/* number of mapped entries */

	void (*scsi_done) (struct scsi_cmnd *);
	dma_addr_t dma_addr;	/* for single DMA (Non-SGL) entry */
	int abort_acount;
} CMD_INFO_T;

/****************************************************************************/
/*                                                                          */
/* NOTE:   SCSI to SATA mapping is as follows                               */
/*                                                                          */
/*     scsi_host       <=>     adapter (the entire HBA)                     */
/*   channel / bus     <=>     port                                         */
/*  target/scsi_device <=>     id 0. The ID can be greater than 0 with      */
/*                             port multipliers                             */
/*        LUN          <=>     0. We don't support LUN > 0                  */
/*                                                                          */
/****************************************************************************/

typedef struct device_info_s {
/** Private Data members - Should NOT be set directly, but these can be    **/
/** read anywhere in the code after they are initialized                   **/
	CMD_INFO_T commands[MAX_COMMANDS];	/* accessed ONLY via SCpnt->host_scribble */
	DDT_T ddt_orig[MAX_TCQ_DEPTH];	/* Array of allocated DMA Descriptor tables  */
	DDT_T ddt[MAX_TCQ_DEPTH];	/* DDT's aligned to 64K physical boundaries  */
	void *vector_table_orig;	/* pointer to memory allocated for VT */
	dma_addr_t vector_table_orig_phy;	/* bus address */
	DMA_VT_T vector_table;	/* The vector table to hold ddts for TCQ */
	//dma_addr_t vector_table_phy;   /* bus address */

/** Public Data members **/

	int dev_tag;		/* driveTag for the associated sata-lib drive object       */
	int adapter_tag;	/* adapterTag for the associated sata-lib adapter object   */
	int port_tag;		/* portTag for the associated sata-lib port object            */
//    int commands_free;            /* Initialized to 0.*/ // NPG: Not being used. cmd_count performs this function.
	struct list_head tcq_free;	/* List of commands (CMD_INFO_T) currently unused */
	spinlock_t free_qlock;
	SATA_FEATURES_T drive_features;	/* List of features enabled for this drive   */
	IDENTIFY_DEVICE_T dev_identify;	/* Copy of Device Identify data of Drive     */
	unsigned int capacity;	/* Capacity of the drive in number of sectors        */
	atomic_t busy;
	atomic_t cmd_count;
	int max_ddt_entries;	/* initialized during gd31244_device_init */
#ifdef SATA_DEBUG_0
	unsigned int max_ddte_count_seen;	/* The maximum number of DDT entries used */
	unsigned int min_ddte_count_seen;	/* The minimum number of DDT entries used */
    ulong all_cmd_seen_count; /* Count of number of commands processed */
#endif
    int online;
} DEVICE_INFO_T;

typedef struct host_info_s {
    struct Scsi_Host *next; /* Pointer to the next Artisea SCSI Host                 */
    int adapter_tag;        /* adapterTag for the associated sata-lib adapter object */
    int adapter_number;     /* 0-n, This is the nth Artisea Adapter found...         */
    struct pci_dev *pdev;   /* Which PCI device ?                                    */
    u32 bar0_raw;           /* Lower Dword for BAR0 in DPA mode                      */
    void *bar0_remapped;    /* ioremapped address of bar0                            */
    char info_string[GD31244_INFO_SZ];  /* Info string used by info API              */
    int dev_tags[MAX_DEVICES_PER_ADAPTER];  /* devTags of connected devices          */
    struct scsi_device *dev_sdev[MAX_DEVICES_PER_ADAPTER];  /* sdev pointers of connected devices */
  int removed;
} HOST_INFO_T;

/* For Debug Dumps */
extern char dump_buffer[DUMP_BUFFER_SZ];

extern SATA_FEATURES_T master_features_g;
#endif				/* GD31244_H */
