/*******************************************************************************
  Copyright (c) 2003-2005, Intel Corporation
  All rights reserved.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License version 2 as published
  by the Free Software Foundation. Alternatively, this software may be
  distributed under the terms of the BSD license. Please see the accompanied
  file COPYING for copyright licensing terms.
*******************************************************************************/
#ifndef GD31244_LLD_H
#define GD31244_LLD_H

#include <linux/version.h>
// Note: This driver is for Linux ver 2.6.0 and higher...
//      Complain if this is not true...
#if (LINUX_VERSION_CODE<KERNEL_VERSION(2,6,0))
   #error "This driver is for Kernel ver 2.6.0 and above only."
#endif

/******************************************************************************/
/*                                                                            */
/*                  I N C L U D E   S T A T E M E N T S                       */
/*                                                                            */
/******************************************************************************/

#include <sata_types.h>     /* Common data types used in SATA driver */
#include <sata.h>           /* Defines for SATA/ATA */
#include <sata_adapter.h>   /* SATA Adapter APIs */
#include <sata_device.h>    /* SATA Drive APIs   */
#include <sata_lib.h>       /* MAX_TCQ_DEPTH, MAX_PORTS_PER_CONTROLLER*/

/* SCSI Includes */
#include <scsi/scsi.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_device.h>

#include <linux/mm.h>       /* Required for scsi_cmnd.h */
#include <asm/scatterlist.h>/* Required for scsi_cmnd.h */
#include <scsi/scsi_cmnd.h> /* Required for gd31244_lld.c */

/* Linux Includes */
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/config.h>
#include <linux/init.h>
#include <linux/pci_ids.h> /* PCI_DEVICE_ID_INTEL_ARTISEA, PCI_VENDOR_ID_INTEL */
#include <linux/interrupt.h>
#include <asm/types.h>
#include <linux/proc_fs.h>
#include <linux/workqueue.h>

/******************************************************************************/
/*                                                                            */
/*                              M A C R O S                                   */
/*                                                                            */
/******************************************************************************/

//
// Debug Macros
//

#ifdef SATA_DEBUG_0
// This macro dumps the error information.
// pdev is a pointer to the pci_dev struct
// dev_tag is the device Tag associated with the drive
#define ERROR_DUMP(pdev, dev_tag)                                                   \
{                                                                                   \
      u32 cr04_status = 0;                                                          \
      u32 crA8_intr = 0;                                                            \
      u32 crAC_intr_mask = 0;                                                       \
                                                                                    \
                                                                                    \
      pci_read_config_dword(pdev, 0x04, &cr04_status);                              \
      pci_read_config_dword(pdev, 0xA8, &crA8_intr);                                \
      pci_read_config_dword(pdev, 0xAC, &crAC_intr_mask);                           \
                                                                                    \
                                                                                    \
      printk(KERN_ERR "\n++++++++++++++++ %s#%d ++++++++++++++++++++\n", __FUNCTION__, __LINE__); \
      if (dev_tag != INVALID_TAG)                                                    \
      {                                                                              \
         DumpErrStat( dev_tag , DUMP_BUFFER_SZ, dump_buffer);                        \
         printk(KERN_ERR "%s\n",dump_buffer);                                        \
      }                                                                              \
                                                                                     \
      printk(KERN_ERR "PCI Config status/cmd %#08x\n",cr04_status);                  \
      printk(KERN_ERR "PCI Config interrupts %#08x\n",crA8_intr);                    \
      printk(KERN_ERR "PCI Config Intr Mask  %#08x\n",crAC_intr_mask);               \
                                                                                     \
                                                                                     \
      printk(KERN_ERR "+++++++++++++++++++++++++++++++++++++++++++\n\n");            \
}
#else
    #define ERROR_DUMP(pdev, dev_tag)
#endif

/* Convert to printable character */
#define PRINT_CHAR(ch) (((ch) < ' ') ? '.' : ( ((ch) > '~') ? '.' : (ch)))

//
// SCSI Section 
//

#define MAX_SECTORS_ADDR 0xffff

#define GD31244_TEMPLATE                                                 \
{                                                                        \
    module:			                   THIS_MODULE,                      \
    name:                              "GD31244",                        \
    detect:           /* Obsolete */   NULL,                   \
    release:          /* Obsolete */   NULL,                  \
    info:                              gd31244_info,                     \
    ioctl:                             NULL, /*gd31244_ioctl, */                   \
	queuecommand:                      gd31244_queuecommand,             \
    eh_abort_handler:                  gd31244_abort,                    \
	eh_device_reset_handler:           gd31244_device_reset,             \
	eh_bus_reset_handler:              gd31244_bus_reset,                \
	eh_host_reset_handler:             NULL,                             \
	slave_alloc:           /* 2.6 */   gd31244_slave_alloc,              \
	slave_configure:       /* 2.6 */   gd31244_slave_configure,          \
	slave_destroy:         /* 2.6 */   gd31244_slave_destroy,            \
	bios_param:                        gd31244_biosparam,                \
	proc_info:        /* Obsolete */   gd31244_proc_info,                \
	proc_name:                         "gd31244",                        \
    proc_dir:                          NULL,						     \
	can_queue:                         GD31244_CANQUEUE,                 \
	this_id:                           -1,                               \
    sg_tablesize:                      256,                              \
    max_sectors:                       MAX_SECTORS_ADDR,                 \
    dma_boundary:           /* 2.6 */  0xffffUL, /* 64K */               \
    cmd_per_lun:                       GD31244_DEVCANQUEUE,              \
    unchecked_isa_dma:                 0,                                \
    use_clustering:                    ENABLE_CLUSTERING,                \
    emulated:                          1,                                \
}

#define ARTISEA_FEATURES                                                 \
{                                                                        \
    FALSE,  /* ReadMultiple */                                           \
    TRUE,   /* MultiWordDMA */                                           \
    32,     /* QUEUE DEPTH */                                            \
    TRUE,   /* ATA API6   */                                             \
    FALSE,  /* PIO: READ BUFFER */                                       \
    FALSE,  /* PIO: WRITE BUFFER */                                      \
    FALSE,  /* HostProtectedArea - .dh. May need when do SoftRAID */     \
    FALSE,  /* DeviceResetCmd - Can do this other ways in Windows.*/     \
    TRUE,   /* Write Cache Enable */                                     \
    TRUE,   /* LookAhead */                                              \
    FALSE,  /* PwrUpStandby */                                           \
    FALSE,  /* SecurityMode */                                           \
    FALSE,  /* SMARTMode */                                              \
    TRUE,   /* Mandatory PM */                                           \
    TRUE,   /* Mandatory FlushCache */                                   \
    TRUE,   /* FlushCache Extended */                                    \
    TRUE,   /* LBA 48 */                                                 \
    FALSE,  /* AcousticMgt */                                            \
    FALSE,  /* SetMaxSecurity */                                         \
    FALSE,   /* Set Features Spinup */                                   \
    FALSE,  /* AdvPowerMgt */                                            \
    FALSE,  /* CFASupported */                                           \
    FALSE,  /* DownLoadMicro */                                          \
    FALSE,  /* Smart Self Test */                                        \
    FALSE,  /* Smart Error Log */                                        \
    TRUE,   /* Media Serial Number */                                    \
    7,      /* DMA Mode */                                               \
/* NCQ should NOT be used due to compatibility issues. See errata. */    \
    FALSE   /* Native Command Queuing. Leave OFF, see below */           \
}


/******************************************************************************/
/*                                                                            */
/*                  M O D E     P A G E S                                     */
/*                                                                            */
/******************************************************************************/
#if 0
#pragma pack(1)
typedef struct mode_param6_header
{
  u8 data_length;
  u8 media_type;
  u8 dev_specific;
  u8 blk_descriptor_length;
} MODE_PARAM6_HEADER_T;

typedef struct mode_param6_param_pg_header
{
   u8 page_code:6;
   u8 reserved :1;
   u8 page_savable:1;
   u8 page_length;
} MODE_PARAM6_PARAM_PG_HEADER_T;

typedef struct mode_param6_cache_pg
{
   MODE_PARAM6_PARAM_PG_HEADER_T pph;
   u8 rcd :1; // Read Cache Disable
   u8 mf:1;   // multiplication factor
   u8 wce:1;  // Write Cache Enable
   u8 reserved:5;
   u8 wrp:4; // Write rentention priority
   u8 drrp:4; // Demand read retention priority
   u16 dptl; // Disable pre-fetch transfer length
   u16 min_pf; // min pre-fetch
   u16 max_pf; // max prefetch
   u16 max_pfc; // max prefetch ceiling
} MODE_PARAM6_CACHE_PG_T;
#pragma pack()
#endif

/******************************************************************************/
/*                                                                            */
/*                 F U N C T I O N    P R O T O T Y P E S                     */
/*                                                                            */
/******************************************************************************/
extern struct workqueue_struct *gd31244_workqueue;
extern struct work_struct dev_issue_pending;
/* SCSI Data functions */
int gd31244_queuecommand(struct scsi_cmnd *,void (*done)(struct scsi_cmnd *));

/* SCSI Error handling Functions */
int gd31244_abort(struct scsi_cmnd *);
int gd31244_bus_reset(struct scsi_cmnd *);
int gd31244_device_reset(struct scsi_cmnd *);

/* SCSI management functions */
int gd31244_detect(struct scsi_host_template  *);
//int gd31244_release(struct Scsi_Host * hpnt);
int gd31244_slave_alloc(struct scsi_device *);
int gd31244_slave_configure(struct scsi_device *);
void gd31244_slave_destroy(struct scsi_device *);

/* SCSI information functions */
int gd31244_biosparam(struct scsi_device *, struct block_device *,sector_t, int []);
int gd31244_proc_info(struct Scsi_Host *, char *, char **, off_t, int, int);
int gd31244_ioctl(struct scsi_device *, int, void *);
const char * gd31244_info(struct Scsi_Host *);

/* Helper functions */
STATUS_CODE_T gd31244_allocate_ddts(struct scsi_device *scd, int num_ddts);
STATUS_CODE_T gd31244_free_ddts(struct scsi_device *scd);
void gd31244_queue_worker(void * dev_info_ptr);

#endif /* GD31244_LLD_H */

