/*******************************************************************************
  Copyright (c) 2003-2005, Intel Corporation
  All rights reserved.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License version 2 as published
  by the Free Software Foundation. Alternatively, this software may be
  distributed under the terms of the BSD license. Please see the accompanied
  file COPYING for copyright licensing terms.
*******************************************************************************/

/******************************************************************************/
/*                                                                            */
/*                  I N C L U D E   S T A T E M E N T S                       */
/*                                                                            */
/******************************************************************************/
#include <sata_debug.h>
#include <sata_types.h>
#include <linux/kernel.h>

/******************************************************************************/
/*                                                                            */
/*                   M A C R O   D E F I N I T I O N S                        */
/*                                                                            */
/******************************************************************************/

/* Convert to printable character */
#define PRINT_CHAR(ch) (((ch) < ' ') ? '.' : ( ((ch) > '~') ? '.' : (ch)))

/******************************************************************************/
/*                                                                            */
/*       G L O B A L   F U N C T I O N   I M P L E M E N T A T I O N S        */
/*                                                                            */
/******************************************************************************/

char dump_buffer[DUMP_BUFFER_SZ];

/******************************************************************************/
/*                                                                            */
/* DESCRIPTION                                                                */
/*                                                                            */
/* PRECONDITIONS                                                              */
/*                                                                            */
/* POST-CONDITIONS                                                            */
/*                                                                            */
/* PARAMETERS                                                                 */
/*                                                                            */
/* RETURNS                                                                    */
/*                                                                            */
/* ON ERRORS                                                                  */
/*                                                                            */
/******************************************************************************/

#ifdef SATA_DEBUG_0

void debug_mem_dump(char * name, char *mem, int size)
{
    int ctr, i, j, print_sz = 16;
    int *u32_ptr = (int *)mem;
    char ch[16];
    char buff[80], *buff_ptr;

    for(i = 0; i < 16; i++) ch[i] = '.' ;

    // Forcing long prints
    //    print_sz = (size > 40) ? 16 : 4;

    if(name == 0)
    {
        name = &ch[3];
    }
    else
    {
        CONSOLE_PRINT("Dumping %s (%d bytes @ %p)\n", name, size , mem);
    }
    ctr = size - (size % print_sz);

    if(( size < 1 ) || ( mem == 0))
    {
        CONSOLE_PRINT("\n*** SORRY. Can't dump location.size = %d, addr = %p\n", size, mem);
        return;
    }

    if(print_sz == 16)
    {
        CONSOLE_PRINT("Byte:  F E D C  B A 9 8  7 6 5 4  3 2 1 0 FEDCBA9876543210\n");
    }
    else
    {
        CONSOLE_PRINT("Byte:  3 2 1 0 3210\n");
    }

    for(i = 0; i < ctr ; i+=print_sz)
    {
        buff_ptr = buff;
        buff_ptr += sprintf(buff_ptr, "[%3d] ", i);
        for(j = print_sz/4 -1 ; j >= 0; j--)

            buff_ptr += sprintf(buff_ptr, "%08x ", u32_ptr[ i/4 + j]);

        for(j = print_sz - 1; j >= 0; j--)
            buff_ptr += sprintf(buff_ptr, "%c", PRINT_CHAR(mem[i+j]));

        CONSOLE_PRINT("%s\n", buff);
    }

    if(size % print_sz)
    {
        buff_ptr = buff;
        buff_ptr += sprintf(buff_ptr, "[%3d] ", ctr);

        for(j = 0; j < (print_sz - (size % print_sz)) / 4 ; j++)
            buff_ptr += sprintf(buff_ptr, "%08x ", 0 );

        for(j =  ((size % print_sz) + 3) / 4 - 1; j >= 0; j--)
            buff_ptr += sprintf(buff_ptr, "%08x ", u32_ptr[ i/4 + j]);

        for(j = (size % print_sz) - 1; j >= 0; j--)
            ch[j] =  mem[ctr+j];

        for(j = print_sz - 1; j >= 0; j--)
            buff_ptr += sprintf(buff_ptr, "%c", PRINT_CHAR(ch[j]));

        CONSOLE_PRINT("%s\n", buff);
    }

}
#endif

