/*******************************************************************************
  Copyright (c) 2003-2005, Intel Corporation
  All rights reserved.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License version 2 as published
  by the Free Software Foundation. Alternatively, this software may be
  distributed under the terms of the BSD license. Please see the accompanied
  file COPYING for copyright licensing terms.
*******************************************************************************/

/******************************************************************************/
/*                                                                            */
/*   DESCRIPTION                                                              */
/*                                                                            */
/*         This file provides the declarations of the various debug           */
/*         macros used in the library                                         */
/*                                                                            */
/******************************************************************************/

#ifndef SATA_DEBIUG_HEADER
#define SATA_DEBIUG_HEADER


/******************************************************************************/
/*                                                                            */
/*                  I N C L U D E   S T A T E M E N T S                       */
/*                                                                            */
/******************************************************************************/

#include <sata_types.h> /* Common data types used in SATA driver */
#include <linux/kernel.h> /* printk */

/******************************************************************************/
/*                                                                            */
/*                   M A C R O   D E F I N I T I O N S                        */
/*                                                                            */
/******************************************************************************/
/*
Key :

Lower level prints will enable all higher level prints

SATA_DEBUG_4    Lower level 4 prints - OSL
SATA_DEBUG_3    Lower level 3 prints - Ports
SATA_DEBUG_2    Lower level 2 prints - Controllers
SATA_DEBUG_1    Lower level 1 prints - Adapter , Device
SATA_DEBUG      General Debug statements & OSDM

*/

#ifdef SATA_DEBUG_4  /* Lower level 4 prints (OSL) */
#  define DEBUG_PRINT_4 DEBUG_ERROR
#  define DEBUG_TRACE_4 DEBUG_TRACE
#  define SATA_DEBUG_3
#endif

#ifdef SATA_DEBUG_3  /* Lower level 3 prints (Ports) */
#  define DEBUG_PRINT_3 DEBUG_PRINT
#  define DEBUG_TRACE_3 DEBUG_TRACE
#  define SATA_DEBUG_2
#endif

#ifdef SATA_DEBUG_2  /* Lower level 2 prints (controller) */
#  define DEBUG_PRINT_2 DEBUG_PRINT
#  define DEBUG_TRACE_2 DEBUG_TRACE
#  define SATA_DEBUG_1
#endif

#ifdef SATA_DEBUG_1  /* Lower level 1 prints (Adapter) */
#  define SATA_DEBUG
#  define DEBUG_PRINT_1 DEBUG_PRINT
#  define DEBUG_TRACE_1 DEBUG_TRACE
#  ifndef SANITY_CHECK
#     define SANITY_CHECK
#  endif
#endif


#ifdef SATA_DEBUG
#define SATA_DEBUG_0
#define DEBUG

//#warning Enabling Debug Macros...

#define CONSOLE_PRINT printk

/* Must have format and at least 1 argument !!!. */
/* If you don't have one, use `%s", ""` to end your string.*/
#  define DEBUG_PRINT(fmt, x...) \
  { \
    printk("%d %s#%d:%s: " fmt "\n", sataDebugCounter++, __FILE__, __LINE__, __FUNCTION__, ##x);\
  }
#  define DEBUG_ERROR(fmt, x...) \
  { \
    printk(KERN_ERR "%d %s#%d:%s: " fmt "\n", sataDebugCounter++, __FILE__, __LINE__, __FUNCTION__, ##x);\
  }

#  define DEBUG_TRACE() \
  { \
    printk("%d %s#%d:%s Function called\n", sataDebugCounter++, __FILE__, __LINE__, __FUNCTION__);\
  }

#  define WARN_STUB() \
  { \
    printk("%d %s#%d:%s Stub Function Called.\n", sataDebugCounter++, __FILE__, __LINE__, __FUNCTION__);\
  }
  
#  define PING() \
  { \
    printk(KERN_ERR "%s#%d:%s ping\n", __FILE__, __LINE__, __FUNCTION__);\
  }

    #define DUMP_VAR(var) debug_mem_dump( #var, (char *) &(var), sizeof(var) );
    #define DEBUG_PRINT_bool(var)  {CONSOLE_PRINT("%s = %s\n", #var,((var == 0) ? "False": "True") );}
    #define DEBUG_PRINT_ch(var)  {CONSOLE_PRINT("%s = `%c` (%d)\n", #var,(char)(var),(var));}
    #define DEBUG_PRINT_int(var) {CONSOLE_PRINT("%s = %d\n",        #var,(var));}
    #define DEBUG_PRINT_str(var) {CONSOLE_PRINT("%s = \"%s\"\n",    #var,( (var)== 0 ? "-NULL-":(var) ));}
    #define DEBUG_PRINT_ptr(var) {CONSOLE_PRINT("%s = %p\n",        #var,(var));}
    #define DEBUG_PRINT_var(var,fmt) {CONSOLE_PRINT("%s = " fmt "\n", #var,(var)); /*debug_mem_dump(0, (char *)&var, sizeof(var));*/}
    #define DEBUG_PRINT_dbl(var) {CONSOLE_PRINT("%s = %f \b",#var,(var));/*debug_mem_dump(0, (char *)&var, sizeof(var)); */}
#else
    
    #define CONSOLE_PRINT(parm...)
    #define DUMP_VAR(var)
    #define DEBUG_PRINT_bool(var)
    #define DEBUG_PRINT_ch(var)
    #define DEBUG_PRINT_int(var)
    #define DEBUG_PRINT_str(var)
    #define DEBUG_PRINT_ptr(var)
    #define DEBUG_PRINT_var(var)
    #define DEBUG_PRINT_dbl(var)
    #define PING()
//  #define DEBUG_ERROR(fmt, x...)
    #define DEBUG_ERROR(fmt, x...) \
  { \
    printk(KERN_ERR "%s#%d:%s: " fmt "\n", __FILE__, __LINE__, __FUNCTION__, ##x);\
  }

#endif /* SATA_DEBUG */

#ifdef SATA_DEBUG_0  /* Hard to reproduce error messages */
#  define DEBUG_ERROR_0(fmt, x...) \
  { \
    printk( KERN_ERR /* KERN_INFO*/  "%s#%d:%s: " fmt "\n", __FILE__, __LINE__, __FUNCTION__, ##x);\
  }

#  define DEBUG_TRACE_0() \
  { \
    printk("%s#%d:%s Function called\n", __FILE__, __LINE__, __FUNCTION__);\
  }

/* How do we assert in device drivers ? */
#define ASSERT(cond) \
  { \
    if( !(cond) ) \
      DEBUG_ERROR("ASSERT Failed (\"%s\")", #cond );\
  }
#endif

/* Define as NULL if not already defined above */
#ifndef DEBUG_PRINT_4
#define DEBUG_PRINT_4(fmt, x...)
#endif
#ifndef DEBUG_PRINT_3
#define DEBUG_PRINT_3(fmt, x...)
#endif
#ifndef DEBUG_PRINT_2
#define DEBUG_PRINT_2(fmt, x...)
#endif
#ifndef DEBUG_PRINT_1
#define DEBUG_PRINT_1(fmt, x...)
#endif
#ifndef DEBUG_PRINT
#define DEBUG_PRINT(fmt, x...)
#endif
#ifndef DEBUG_TRACE_4
#define DEBUG_TRACE_4()
#endif
#ifndef DEBUG_TRACE_3
#define DEBUG_TRACE_3()
#endif
#ifndef DEBUG_TRACE_2
#define DEBUG_TRACE_2()
#endif
#ifndef DEBUG_TRACE_1
#define DEBUG_TRACE_1()
#endif
#ifndef DEBUG_TRACE
#define DEBUG_TRACE()
#endif
#ifndef WARN_STUB
#define WARN_STUB()
#endif
#ifndef ASSERT
#define ASSERT(cond)
#endif
#ifndef DEBUG_ERROR_0
#define DEBUG_ERROR_0(fmt, x...)
#endif
#ifndef DEBUG_TRACE_0
#define DEBUG_TRACE_0()
#endif

#define MAX(a,b) ( (a) < (b) ? (b) : (a) )
#define MIN(a,b) ( (a) > (b) ? (b) : (a) )

/******************************************************************************/
/*                                                                            */
/*    V A R I A B L E   A N D   S T R U C T U R E   D E F I N I T I O N S     */
/*                                                                            */
/******************************************************************************/

#ifdef SATA_DEBUG_0
   #define DUMP_BUFFER_SZ 4096
#else
   #define DUMP_BUFFER_SZ 80
#endif

extern char dump_buffer[DUMP_BUFFER_SZ];

#ifdef SATA_DEBUG
extern INT sataDebugCounter;
#endif


/******************************************************************************/
/*                                                                            */
/*                 F U N C T I O N    P R O T O T Y P E S                     */
/*                                                                            */
/******************************************************************************/

#ifdef SATA_DEBUG_0
// During Debug, Use this function to dump out memory contents.
void debug_mem_dump(char * name, char *mem, int size);
#else
// In production drivers, define it as a NULL macro.
#define debug_mem_dump(name_p, mem_p, size)
#endif

#endif /* SATA_DEBIUG_HEADER */
