/*******************************************************************************
  Copyright (c) 2003-2005, Intel Corporation
  All rights reserved.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License version 2 as published
  by the Free Software Foundation. Alternatively, this software may be
  distributed under the terms of the BSD license. Please see the accompanied
  file COPYING for copyright licensing terms.
*******************************************************************************/

/******************************************************************************/
/*                                                                            */
/*                  I N C L U D E   S T A T E M E N T S                       */
/*                                                                            */
/******************************************************************************/
#include <OSL.h>
#include <sata_lib.h> /* Definations of structures internal to SATA Lib */

/******************************************************************************/
/*                                                                            */
/*       G L O B A L   F U N C T I O N   I M P L E M E N T A T I O N S        */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/*                                                                            */
/* DESCRIPTION      This function traverses an array of pointers looking for  */
/*                  the next position containing a NULL_POINTER.              */
/*                                                                            */
/* PRECONDITIONS    The array has been initialized to all NULL_POINTER values */
/*                  before first use.                                         */
/*                                                                            */
/* POST-CONDITIONS                                                            */
/*                                                                            */
/* PARAMETERS                                                                 */
/*          pointerArray_i  Array of pointers. Most likely either ports_gp,   */
/*                          adapters_gp, controllers_gp or devices_gp         */
/*          max_i           Number of elements in the array. Most likely      */
/*                          either MAX_ADAPTERS, MAX_CONTROLLERS, MAX_PORTS   */
/*                          or MAX_DEVICES                                    */
/*                                                                            */
/* RETURNS                                                                    */
/*          Index number of next NULL pointer if < max else INVALID_TAG       */
/*                                                                            */
/******************************************************************************/

INT getNextSlot(void ** pointerArray_i, INT max_i)
{
    INT ctr;

    /* check parameters */
    if(pointerArray_i == NULL_POINTER || max_i <= 0)
    {
        return INVALID_TAG;
    }

    for(ctr = 0; ctr < max_i; ctr++)
    {
        if(pointerArray_i[ctr] == NULL_POINTER)
        {
            return ctr;
        }
    }

    /* Looked at the entire array but found no NULL_POINTER. Return INVALID_TAG */
    return INVALID_TAG;
}
