/*******************************************************************************
  Copyright (c) 2003-2005, Intel Corporation
  All rights reserved.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License version 2 as published
  by the Free Software Foundation. Alternatively, this software may be
  distributed under the terms of the BSD license. Please see the accompanied
  file COPYING for copyright licensing terms.
*******************************************************************************/

/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*   DESCRIPTION                                                              */
/*                                                                            */
/*       This file contains the definations of standard types to be used for  */
/*       development of the SATA code. The purpose of using these types       */
/*       throughout the program instead of the standard C data types is       */
/*       for easier portability to different architectures and OSes.          */
/*                                                                            */
/******************************************************************************/

#ifndef SATA_TYPES_HEADER
#define SATA_TYPES_HEADER


/******************************************************************************/
/*                                                                            */
/*                  I N C L U D E   S T A T E M E N T S                       */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/*                                                                            */
/*                C O N S T A N T   D E F I N I T I O N S                     */
/*                                                                            */
/******************************************************************************/

#define DDT_MAX_ENTRIES   (8192) /* DDT can be max of 64Kb, 8 Bytes per entry */
#define DDT_ENTRY_EOT_BIT (15)   /* DMA Descriptor Table Entry EOT bit        */
#define DDT_ENTRY_MAX_COUNT (0xfffe) /* Even number of bytes, max 64K         */

/* Artisea DMA Specific Bit positions are defined in artisea.h */


/******************************************************************************/
/*                                                                            */
/*                   M A C R O   D E F I N I T I O N S                        */
/*                                                                            */
/******************************************************************************/
/*
 * All pointers should be initialized to the following macro
 * if there is no other value that it should be initialized too.
 * This will allow the pointer initialization value to be
 * easily changed to an address that can cause an exception
 * to aid debugging of runaway pointers.
 */
#ifndef NULL_POINTER
#define NULL_POINTER (void *)(0)
#endif

/******************************************************************************/
/*                                                                            */
/*                    T Y P E   D E F I N I T I O N S                         */
/*                                                                            */
/******************************************************************************/

/* Type defines shared with other driver code */

/*************************************/
/*  common to all OSes               */
/*************************************/

#ifdef  __OOOLD__

typedef char            CHAR;
typedef unsigned char   UCHAR;

#ifndef INT8
typedef signed char     INT8;
#endif
#ifndef UINT8
typedef unsigned char   UINT8;
#endif
#ifndef INT16
typedef short           INT16;
#endif
#ifndef UINT16
typedef unsigned short  UINT16;
#endif
typedef short           SHORT;
typedef unsigned short  USHORT;

typedef int             INT;
typedef unsigned int    UINT;

#endif

/*************************************/
/* specific                          */
/*************************************/

/*************************************/
/* Start WinNT specific              */
/*************************************/
#ifdef WINNT

//#include "stddef.h"
#define OS_SPINLOCK_T INT

typedef char            CHAR;
typedef unsigned char   UCHAR;

typedef short           SHORT;
typedef unsigned short  USHORT;

typedef int             INT;
typedef unsigned int    UINT;

typedef int             INT32;
typedef unsigned int    UINT32;

typedef long            LONG;
typedef unsigned long   ULONG;

#ifdef __64BIT__

/* IA64: int, long remain 32bit; __int64, pointer 64bit */
typedef int             LONG32;     /* longs in MS are always 32bit */
typedef unsigned int    ULONG32;    /* in IA64 set to int, because MS includes */
/* define it this way. */
#else
typedef int             LONG32;     /* longs in MS are always 32bit */
typedef unsigned int    ULONG32;
#endif

#ifdef __64BIT__

/* IA64: int, long remain 32bit; __int64, pointer 64bit */
typedef unsigned __int64    ULONG_PTR;  /* ULONG with size of pointer */
#else
typedef unsigned long   ULONG_PTR;
#endif

/* not defined by win ?? */
typedef signed char     INT8;
typedef unsigned char   UINT8;
typedef short           INT16;
typedef unsigned short  UINT16;
typedef unsigned __int64			UINT64;


/*
typedef struct
{
	UINT32  Lo;
	UINT32  Hi;

} UINT64;
*/

#ifndef _WINDOWS_
typedef unsigned char   boolean;
#endif

#endif

/*************************************/
/* End WinNT specific                */
/*************************************/

/*************************************/
/* Start Netware specific            */
/*************************************/
#ifdef __NETWARE__
typedef char            CHAR;
typedef signed char     INT8;
#ifndef UCHAR
typedef unsigned char   UCHAR;
#endif
typedef unsigned char   UINT8;

typedef short           INT16;
typedef unsigned short  UINT16;
typedef short           SHORT;
typedef unsigned short  USHORT;

typedef int             INT;
typedef unsigned int    UINT;

typedef int             INT32;
typedef unsigned int    UINT32;

typedef long            LONG32;
typedef unsigned long   ULONG32;

/*typedef long		    LONG;*/
typedef unsigned long   ULONG;

typedef unsigned long   ULONG_PTR;

#ifndef boolean
typedef int             boolean;
#endif

#endif

/*************************************/
/* End Netware specific              */
/*************************************/

/*************************************/
/* Start Unix specific               */
/*************************************/
// #ifdef __unix__
#ifdef UNIX

typedef char            CHAR;
typedef signed char     INT8;
#ifndef UCHAR
typedef unsigned char   UCHAR;
#endif
typedef unsigned char   UINT8;

typedef short           INT16;
typedef unsigned short  UINT16;
typedef short           SHORT;
typedef unsigned short  USHORT;

typedef int             INT;
typedef unsigned int    UINT;

typedef int             INT32;
typedef unsigned int    UINT32;

typedef unsigned long long UINT64;

#ifdef __64BIT__

/* Unix' longs scale with pointer size (=architecture size). ulong 64bit */
typedef int             LONG32;
typedef unsigned int    ULONG32;

#else

/* Unix 32bit */

typedef long            LONG32;
typedef unsigned long   ULONG32;

#endif

typedef long            LONG;
typedef unsigned long   ULONG;

typedef unsigned long   ULONG_PTR;

#ifndef boolean
typedef int             boolean;
#endif

#endif

/*************************************/
/* End Unix specific                 */
/*************************************/

#ifndef __linux__
typedef INT FLAGS_T;
#else
typedef unsigned long FLAGS_T;
#endif

#ifdef __linux__

#include <linux/types.h>
#ifndef __KERNEL__
#define __KERNEL__
#endif
typedef s8            CHAR;
typedef s8            INT8;
#ifndef UCHAR
typedef u8            UCHAR;
#endif
typedef u8            UINT8;

typedef s16           INT16;
typedef u16           UINT16;
typedef s16           SHORT;
typedef u16           USHORT;

typedef int             INT;
typedef unsigned int    UINT;

typedef s32             INT32;
typedef u32             UINT32;

typedef s32             LONG;
typedef u32             ULONG;

typedef u64             UINT64;
typedef s64             INT64;

#ifndef boolean
typedef int             boolean;
#endif

#endif


/*************************************/
/* Start OS/2 specific               */
/*************************************/
#ifdef __OS2__

#if !defined(__OS2DEF__) && !defined(OS2DEF_INCLUDED)
typedef char            CHAR;
typedef signed char     INT8;
typedef unsigned char   UCHAR;
typedef unsigned char   UINT8;

typedef short           INT16;
typedef unsigned short  UINT16;
typedef short           SHORT;
typedef unsigned short  USHORT;

typedef int             INT;
typedef unsigned int    UINT;

#endif

typedef int             INT32;
typedef unsigned int    UINT32;

typedef long            LONG32;
typedef unsigned long   ULONG32;

/*typedef long		LONG;*/
typedef unsigned long   ULONG;

typedef unsigned long   ULONG_PTR;

typedef int             boolean;

#endif

/*************************************/
/* End OS/2 specific                 */
/*************************************/

/*************************************/
/* Start DOS specific                */
/*************************************/

#if defined(__DOS__) || defined(__ROM__) || defined(__BIOS__)
typedef char            CHAR;
#ifndef INT8
typedef signed char     INT8;
#endif
typedef unsigned char   UCHAR;
#ifndef UINT8
typedef unsigned char   UINT8;
#endif

#ifndef INT16
typedef short           INT16;
#endif
#ifndef UINT16
typedef unsigned short  UINT16;
#endif
typedef short           SHORT;
typedef unsigned short  USHORT;

typedef int             INT;
typedef unsigned int    UINT;

#ifndef INT32
typedef long            INT32;
#endif
#ifndef UINT32
typedef unsigned long   UINT32;
#endif

typedef long            LONG32;
typedef unsigned long   ULONG32;

#ifndef LONG
typedef long            LONG;
#endif
typedef unsigned long   ULONG;

typedef unsigned long   ULONG_PTR;

typedef int             boolean;

#endif

/*************************************/
/* End DOS specific                  */
/*************************************/


/* new type defines: numbers indicate bytes, not bits */
#define	INT1	INT8
#define UINT1	UINT8
#define INT2	INT16
#define UINT2	UINT16
#define INT4	INT32
#define UINT4	UINT32
#ifndef FALSE
#   define FALSE (0)
#   define TRUE  (!FALSE)
#endif

#if 0

/* Stop using these... These will be replaced soon. - NPG
size of char = 1
size of short = 2
size of int = 4
size of long = 4
size of long long = 8
size of char pointer = 4

typedef unsigned char      UINT8_T;
typedef unsigned short     UINT16_T;
typedef unsigned long      UINT32_T;
typedef unsigned long long UINT64_T;

typedef signed char        SINT8_T;
typedef signed short       SINT16_T;
typedef signed long        SINT32_T;
typedef signed long long   SINT64_T;
typedef float              F32_T;
typedef double             F64_T;

/ *
 * For Booleans, one can use the typedef enum below.
 * It is advised to utilize enums over Booleans to allow
 * for improved compiler checking
 */
#undef TRUE
#undef FALSE
typedef enum BOOL
{
   FALSE = 0,
   TRUE = !FALSE
} BOOL_T;

#endif  /* #if 0 */

/* The Addresses for Registers on the Artisea Controller */
typedef UINT8 * SATA_REG_ADDR_T;
/* Bus addresses of locations as seen by a DMAC */
typedef UINT32 BUS_ADDR_T;

/******************************************************************************/
/* This type will define the return code values that the SATA functions       */
/* will return.  Success and failure are predefined.  Any other error code    */
/* should be added to this enum.                                              */
/******************************************************************************/
typedef enum STATUS_CODE
{
   SATA_SUCCESS               = 0, /* Indicates the function succeeded        */
   SATA_SUCCESS_COMPLETED     = 0, /* Success, operation completed            */
   SATA_SUCCESS_NOT_COMPLETED = 1, /* Success, operation Not completed(PIO...)*/
   SATA_FAILURE               = 2, /* Indicates the Fatal function failed     */
   SATA_COMPLETED_WITH_ERRORS = 3, /* Task completed with non fatal errors    */
   SATA_GET_MEMORY_FAILED     = 4, /* Failed to get system memory             */
   SATA_FREE_MEMORY_FAILED    = 5, /* Failed to free system memory            */
   SATA_OUT_OF_RANGE_VALUE    = 6, /* Our of Range error                      */
   SATA_NOT_INITIALIZED       = 7, /* SATA Object not initialized / present   */
   SATA_BUSY                  = 8, /* Destination was busy                    */
   SATA_FALSE                 = 0, /* Same as FALSE                           */
   SATA_TRUE                  = 9, /* Unique STATUS_CODE. NOT same as TRUE    */
   SATA_HOTPLUG_EVENT		  = 10

} STATUS_CODE_T;


/******************************************************************************/
/*                                                                            */
/* DMA Dedcriptor Table (DDT) Entry structure.                                */
/*     The first DWORD in the entry is the address of a data buffer. Buffers  */
/*     are aligned on WORD boundaries.                                        */
/*     The second field is an even numbered 16 bit byte count of the buffer.  */
/*     The MSB of the last WORD marks the entry as the End-Of-Table entry.    */
/* In a given DDT, two consecutive entries are offset by 16 bytes and are     */
/* aligned on a 4-byte boundary. All entries share the same 4GB page.         */
/*                                                                            */
/******************************************************************************/

typedef struct DDT_ENTRY_S
{
    BUS_ADDR_T baseAddress; /* Base address of Buffer Region                  */
    UINT16   byteCount;   /* Byte count of data Buffer                        */
    UINT16   EOT;       /* Set the DDT_ENTRY_EOT_BIT to 1 on last table entry */
} DDT_ENTRY_T;

/* DDT is managed by OSDM */
/* Note: Zero Copy transfers require that the user allocated buffers be used  */
/* for the data transfers rather than making duplicate copies of the data.    */
/* Some OSes might not support Zero Copy at this time.  Our approach will be  */
/* to let the OS specific deiver decide what to do.  We will just use the     */
/* DDT that is passed down to us.                                             */

typedef struct DDT_S  /* Used by OSDM */
{
    UINT64       ddtPhysicalAddress; /* Physical bus address of the start of the DDT     */
	UINT32		 ddtePage; /* Upper 32 bits of the entries in the DDT */
    DDT_ENTRY_T *ddte;    /* CPU view of DDT */
} DDT_T;

typedef struct DMA_INFO_S /* Used internally by device */
{
    UINT64 ddt;      /* DMA view of the DMA Descriptor Table.     */
    UINT32 ddtePage; /* Top 32 bits of DDT_ENTRY baseAddress      */
    UINT8 status;    /* Copy of the DMA Status Register - SUPDDSR */
} DMA_INFO_T;

typedef struct DMA_VECTORS_S
{
    UINT32      ddtPhysicalAddress;
    //UINT64      ddtPhysicalAddress;
//    UINT32      ddtPhysicalAddressLow;
//    UINT32      ddtPhysicalAddressHigh;
} DMA_VECTORS_T;

typedef struct DMA_VT_S
{
    UINT64       vtPhysicalAddress; /* Physical bus address of the vector table "vt" */
    //DMA_VECTORS_T vt[32];           /* The physical bus address of each DDT */
    DMA_VECTORS_T *vt;
} DMA_VT_T;

#endif /* SATA_TYPES_HEADER */
