/*
 * linux/include/asm-arm/arch-iop3xx/memory.h
 */

#ifndef __ASM_ARCH_MEMORY_H
#define __ASM_ARCH_MEMORY_H

#include <linux/config.h>
#include <asm/hardware.h>


#define PAGE_OFFSET             UL(0x80000000)
#define TASK_SIZE               (PAGE_OFFSET - UL(0x01000000))
#define TASK_UNMAPPED_BASE      ((PAGE_OFFSET) / 3)


/*
 * Physical DRAM offset.
 */
#ifndef CONFIG_ARCH_IOP331
#define PHYS_OFFSET	UL(0xa0000000)
#else
#define PHYS_OFFSET	UL(0x00000000)
#endif

/*
 * Virtual view <-> PCI DMA view memory address translations
 * virt_to_bus: Used to translate the virtual address to an
 *		address suitable to be passed to set_dma_addr
 * bus_to_virt: Used to convert an address for DMA operations
 *		to an address that the kernel can use.
 */
#if defined(CONFIG_ARCH_IOP321)

#define __virt_to_bus(x)	(((__virt_to_phys(x)) & ~(*IOP321_IATVR2)) | ((*IOP321_IABAR2) & 0xfffffff0))
#define __bus_to_virt(x)    (__phys_to_virt(((x) & ~(*IOP321_IALR2)) | ( *IOP321_IATVR2)))

#elif defined(CONFIG_ARCH_IOP331)

#if 1
/*
 * These macros use BAR2 for sharing SDRAM
 * This is the most common configuration but potentially takes a
 * lot of PCI Memory space since the IOP331 supports up to 2GB of SDRAM
 */
#define __virt_to_bus(x)	(((__virt_to_phys(x)) & ~(*IOP331_IATVR2)) | ((*IOP331_IABAR2) & 0xfffffff0))
#define __bus_to_virt(x)    (__phys_to_virt(((x) & ~(*IOP331_IALR2)) | ( *IOP331_IATVR2)))
#else
/*
 * These macros use BAR3 for sharing SDRAM
 * This can use Opaque memory in the IOP331 bridge to avoid having a huge
 * PCI memory window.  This is a difficult configuration however since this
 * requires using 64-bit(DAC) addressing and there are many devices incapable
 * of using DAC for DMA
 */
#define __pci_sdram_base()  (((dma64_addr_t)(*IOP331_IABAR3 &~ 0xf)) | (((dma64_addr_t)*IOP331_IAUBAR3) << 32))
#define __virt_to_bus(x)    (dma64_addr_t)(__virt_to_phys(x) - *IOP331_IATVR3) + __pci_sdram_base()
#define __bus_to_virt(x)    (u32)(x - __pci_sdram_base()) + __phys_to_virt(*IOP331_IATVR3)
#endif

#endif

#endif
